#!/usr/bin/env python
# ======================================================================
# Copyright (C) 2001 and onwards Robert Muth <robert at muth dot org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================

"""
Journal: Winzig's generic data browsing class module
"""

# python imports
import gtk
import gobject

import logging

# local imports
import wutil

__all__ = ["DataBrowser",
           "DataBrowserWithLabels",
           "DataEditorWithForwardBackward"]

# ======================================================================
class DataBrowser:
    def SwitchToEditor(self, rid):
        """This must be defined in the subclass
        """
        raise NotImplementedError, "abstract base class"

    def FixupRow(self, dummy_row):
        """this may be defined by the subclass
        """
        return

    def AddExtraButtons(self, dummy_hbox):
        """this may be defined by the subclass
        """
        return

    def AddExtraRow(self, dummy_vbox):
        """this may be defined by the subclass
        """
        return

    def _ExtractRecordIdsFromModel(self):
        lst = []
        def func(model, dummy_path, it):
            lst.append(model.get_value(it, 0))
            return 0
        self._model.foreach(func)
        return lst

    def _MakeTreeView(self):
        titles = self._db.GetTitles()
        columns = [gobject.TYPE_INT] + [gobject.TYPE_STRING] * len(titles)
        def ActionRowsReordered(dummy):
            logging.info("key reordered")
            # we need to know the new order for the next and prev buttons
            self._current_ids = self._ExtractRecordIdsFromModel()
            return
        self._model = gtk.ListStore(*columns)
        self._model.connect("sort-column-changed", ActionRowsReordered)
        self._treeview = gtk.TreeView(self._model)

        for n, t in enumerate(titles):
            renderer = gtk.CellRendererText()
            renderer.set_property('xalign', 0.0) # left align
            renderer.set_property('yalign', 0.0) # top align
            column = gtk.TreeViewColumn(t, renderer, text=n+1)
            column.set_resizable(1)
            column.set_min_width(10)
            column.set_fixed_width(150)
            #
            column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
            self._treeview.append_column(column)
            column.set_sort_column_id(n+1)

        self._model.set_sort_column_id(1, gtk.SORT_ASCENDING)
        self._treeview.set_search_column(0)
        self._treeview.set_fixed_height_mode(1)

        # potential speed improvement which currently does not work
        # because rows need to be fix width
        #self._treeview.set_fixed_height_mode(1)
        self._treeview.show()

        def ActionEditItem(dummy_arg1, path, dummy_col):
            it = self._model.get_iter(path)
            rid = self._model.get_value(it, 0)
            self.SwitchToEditor(rid)
            return

        self._treeview.connect("row_activated", ActionEditItem)
        return

    def GetCurrentRecordIds(self):
        return self._db.RestrictByLabels([], self._search_button.get_active())

    def Redraw(self):
        # hack to avoid update by ActionRowsReordered()
        current_ids = self.GetCurrentRecordIds()
        logging.info("redrawing %d records", len(current_ids))
        # disconnect model to speed things up
        # c.f. http://www.pygtk.org/pygtk2tutorial/sec-TreeModelInterface.html#sec-AddingStoreRows
        model = self._treeview.get_model()
        self._treeview.set_model(None)
        # save old sorting mode then disable sorting
        col, asc_desc =  model.get_sort_column_id()
        model.set_sort_column_id(-2, gtk.SORT_ASCENDING)
        #model.set_default_sort_func(None)
        model.clear()

        for row in self._db.GetBrowserRecords(current_ids):
            self.FixupRow(row)
            it = model.append(row)

        logging.info("swapping model")
        # enables sorting
        model.set_sort_column_id(col, asc_desc)
        self._treeview.set_model(model)
        logging.info("redraw done")
        self._current_ids = current_ids
        return

    def GetSelectedId(self):
        selection = self._treeview.get_selection()
        (dummy, it) =  selection.get_selected()
        if not it: return -1
        return self._model.get_value(it, 0)


    def GetNextPrevRecord(self, record_id, direction):
#        return record_id
        try:
            index = direction + self._current_ids.index(record_id)
            if index < 0: return record_id   # negative indices do not cause exceptions
            return self._current_ids[index]
        except:
            return record_id

    def AddStandardButtons(self, hbox):
        """
        Add New and Del buttons
        """
        def ActionNew(dummy, dummy2=None):
            self.SwitchToEditor(-1)
            return

        button = wutil.MyButton(_(" New "),
                                _("create a new %s") % self._name,
                                ActionNew)
        hbox.pack_start(button, 0, 0)
        button.show()

        # ======================================================================

        def ActionDel(dummy, dummy2=None):
            def DelRecord():
                rid = self.GetSelectedId()
                if rid < 0: return
                self._db.RecordDelete(rid)
                self.Redraw()
                return

            wutil.gui_message(_("Del"), _("Delete this %s" % self._name),
                              [(_("OK"), DelRecord), (_("CANCEL"), None)])
            return

        button = wutil.MyButton(_(" Del "),
                                _("delete a %s" % self._name),
                                ActionDel)
        hbox.pack_start(button, 0, 0)
        button.show()
        # ======================================================================


    def AddSearch(self, hbox):
        """
        Add Search Entry and Toggle Button
        """

        def ActionReturnPressed(dummy):
            if self._search_button.get_active():
                self._db.MatchRecords(self._search_entry.get_text())
                self.Redraw()
            else:
                self._search_button.set_active(1)  # this will trigger a button event
            return


        entry = wutil.MyEntry(10, 1,
                              _("search records for text/regex (press [Return])"),None)

        entry.show()
        hbox.pack_start(entry, 0, 0)
        entry.connect("activate", ActionReturnPressed)
        self._search_entry = entry

        def ActionSearchToggled(dummy, dummy2=None):
            if self._search_button.get_active():
                self._db.MatchRecords(self._search_entry.get_text())
            self.Redraw()
            return

        button = wutil.MyCheckButton(_("Match"), 0, 1,
                                     _("enable/disable search"),
                                     ActionSearchToggled)
        hbox.pack_start(button, 0, 0)
        button.show()
        self._search_button = button
        return

    def __init__(self, name, database):
        logging.info("databrowser creating widget %s", name)
        self._widget = None
        self._db = database
        self._MakeTreeView()
        self._buttons = []
        self._current_ids = []
        self._name = name
        # ======================================================================

        vbox = wutil.MyVBox()

        # ======================================================================
        ## label selector
        # ======================================================================

        hbox = wutil.MyHBox()
        hbox.show()
        vbox.pack_start(hbox, 0, 0)

        self.AddStandardButtons(hbox)
        self.AddSearch(hbox)
        self.AddExtraButtons(hbox)

        self.AddExtraRow(vbox)
        # ======================================================================
        ## browser
        # ======================================================================

        window = wutil.MyScrolledWindow()
        vbox.pack_start(window)
        window.show()
        window.add(self._treeview)


        # ======================================================================
        vbox.show()
        self._widget = vbox
        logging.info("databrowser widget %s complete", name)
        return

    def get_widget(self):
        return self._widget


# ======================================================================
##
# ======================================================================

class DataBrowserWithLabels(DataBrowser):

    def GetcurrentLabels(self):
        labels = []
        for b in self._buttons:
            if b.get_property('visible'):
                labels.append( b.get_label())
        return labels

    def GetCurrentRecordIds(self):
        current_labels = self.GetcurrentLabels()
        current_ids = self._db.RestrictByLabels(current_labels,
                                                 self._search_button.get_active())
        labels_and_counts = self._db.GetLabelsAndCounts(current_ids)
        keep = [x for x in labels_and_counts if x[0] not in current_labels]
        keep.sort()

        for item in  self._menu_items:
            if keep:
                text = ("%s (%d)" % keep.pop(0))
                item.get_child().set_text(text)
                item.show()
            else:
                item.hide()

        return current_ids


    def AddExtraButtons(self, hbox):
        self._menu_items = []

        def ActionAddLabel(item):
            label = item.get_child().get_text().split()[0]
            if not label:
                print "ignoring empty label"
                return
            free_button = None
            for b in self._buttons:
                if not b.get_property('visible'):
                    free_button = b
                    continue
                if b.get_label() == label:
#                    print "label already exists"
                    return

            if free_button:
                free_button.set_label(label)
                free_button.show()
            self.Redraw()
            return

        menubar = gtk.MenuBar()
        menubar.show()
        hbox.pack_end(menubar, 0, 0)

        menutitle = wutil.MyMenuTitle(_("Restrict"),
                                      _("select a label to limit shown records"))
        menutitle.show()
        menubar.append(menutitle)
        menu = gtk.Menu()
        menutitle.set_submenu(menu)

        for dummy in range(100):
            item = gtk.MenuItem("dummy")   # the string is important (forces label creation)
            item.connect("activate", ActionAddLabel)
            menu.append(item)
            self._menu_items.append(item)

        # ======================================================================

        def ActionDeleteLabel(button, dummy=None):
            button.hide()
            self.Redraw()
            return

        for dummy in range(50):
            button = wutil.MyButton("",
                                    _("click to remove label"),
                                    ActionDeleteLabel)
            hbox.pack_end(button, 0, 0)
            self._buttons.append(button)
        return


# ======================================================================
# EDITOR STUFF
# ======================================================================


class DataEditorWithForwardBackward(object):

    def SwitchToBrowser(self, modified):
        raise NotImplementedError, "abstract base class"

    def CreateEditWidgets(self, vbox):
        raise NotImplementedError, "abstract base class"

    def LoadRecordDataIntoWidgets(self, record):
        raise NotImplementedError, "abstract base class"

    def RetrieveRecordDataFromWidgets(self, record):
        raise NotImplementedError, "abstract base class"

    def GetNextPrevRecord(self, rid, direction):
        raise NotImplementedError, "abstract base class"

    def AddExtraButtons(self, dummy_hbox):
        """this may be defined by the subclass
        """
        return

    def set_dirty(self, dirty):
        if self._ignore_dirty_requests: return
        # prop = 'visible'
        prop = 'sensitive'
        if self._dirty == dirty: return # optimization
        self._dirty = dirty
        for b in self._dirty_buttons:
            b.set_property(prop, dirty)
        for b in self._clean_buttons:
            b.set_property(prop, not dirty)
        wutil.SetDirty(self._name, dirty)
        return

    def _CreateButtonWidgets(self, hbox):
        """
        Create Move, New, Dup, Save, Reset and Back buttons
        """
        def ActionEditorMove(dummy_widget, direction):
            assert( not self._dirty )

            rid = self.GetNextPrevRecord(self._current_id, direction)
            if rid >= 0:
                self.set_edit_record(rid)
            return


        button = wutil.MyButton(" << ",
                                _("previous record"),
                                ActionEditorMove, -1)
        hbox.pack_start(button, 0, 0)
        button.show()
        self._clean_buttons.append(button)

        button = wutil.MyButton(" >> ",
                                _("next record"),
                                ActionEditorMove, +1)
        hbox.pack_start(button, 0, 0)
        button.show()
        self._clean_buttons.append(button)


        # ======================================================================

        def ActionClickedNewEntry(dummy, dummy2=None):
            self.set_edit_record(-1)
            return

        button = wutil.MyButton(_(" New "),
                                _("new note"),
                                ActionClickedNewEntry)
        hbox.pack_start(button, 0, 0)
        button.show()
        self._clean_buttons.append(button)

        # ======================================================================

        def ActionClickedDupEntry(dummy, dummy2=None):
            self.set_edit_record(-1, self._current_record)
            return

        button = wutil.MyButton(_(" Dup "),
                                _("duplicate note"),
                                ActionClickedDupEntry)
        hbox.pack_start(button, 0, 0)
        button.show()
        self._clean_buttons.append(button)

        # ======================================================================

        def ActionUpdate(dummy, dummy2=None):
            try:
                record = self._current_record
                self.RetrieveRecordDataFromWidgets(record)
                if self._current_id < 0:
                    rid = self._db.RecordCreate(record)
                else:
                    self._db.RecordUpdate(self._current_id, record)
                    rid = self._current_id
                self.set_edit_record(rid)
            except Exception, err:
                wutil.gui_error(str(err))
            self._modified = 1
            return

        button = wutil.MyButton(_(" Save "),
                                _("save modified note"),
                                ActionUpdate)
        hbox.pack_start(button, 0, 0)
        button.show()
        self._dirty_buttons.append(button)

        # ======================================================================

        def ActionReset(dummy, dummy2=None):
            self.edit_reset()
            return

        button = wutil.MyButton(_(" Reset "),
                                _("discard modifications"),
                                ActionReset)
        hbox.pack_start(button, 0, 0)
        button.show()
        self._dirty_buttons.append(button)

        # ======================================================================

        label = wutil.MyLabelWithMarkup(_(" <b>New Entry</b> "))
        label.show()
        hbox.pack_start(label, 0, 0)
        self._new_buttons.append(label)

        # ======================================================================

        def ActionBack(dummy, dummy2=None):
            if self._dirty:
                wutil.gui_warning(_("Reset or Save Changes First"))
            else:
                self.SwitchToBrowser(self._modified)
            return

        button = wutil.MyButton(_(" Back "),
                                _("back to notes browser"),
                                ActionBack)
        hbox.pack_end(button, 0, 0)
        button.show()
        self._clean_buttons.append(button)
        return


    def __init__(self, name, database):
        self._name = name
        self._ignore_dirty_requests = 0
        # if this is 1 we trigger a redraw when swicthing back
        self._modified = 0
        self._db = database
        self._widget = None
        # neither 0 or 1, this is important for set_dirty optimization
        self._dirty = None
        # make sure thing a clean when reloading and app
        wutil.SetDirty(self._name, 0)

        self._current_id = None
        self._current_record = None

        # record widgets:
        self._buttons = []

        self._dirty_buttons = []
        self._clean_buttons = []
        self._new_buttons = []

        vbox = wutil.MyVBox()

        # buttons
        hbox = wutil.MyHBox()
        vbox.pack_start(hbox, 0, 0)
        hbox.show()

        self._CreateButtonWidgets(hbox)
        self.AddExtraButtons(hbox)

        self.CreateEditWidgets(vbox)
        vbox.show()
        self._widget = vbox
        return

    def get_widget(self):
        return self._widget

    def edit_reset(self):
        """
        """
        # we do not want the following statesments to fire dirty events
        self._ignore_dirty_requests = 1
        self.LoadRecordDataIntoWidgets(self._current_record)
        self._ignore_dirty_requests = 0
        self.set_dirty(0)
        return

    def set_edit_record(self, rid, record = None):
        self._current_id = rid
        if record:
            self._current_record = record
        else:
            self._current_record = self._db.RecordLookup(rid)
        for b in self._new_buttons:
            b.set_property('visible', rid < 0)
        self.edit_reset()
        return

