#!/usr/bin/python
# ======================================================================
# Copyright (C) 2001 and onwards Robert Muth <robert at muth dot org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================
"""
Simple geo-code annotator for winzig's address book

Note:
This program uses Google's geocoding service, to apply for a key
http://www.google.com/apis/maps/signup.html
http://www.google.com/apis/maps/documentation/index.html#Geocoding_HTTP_Request
 To update geo codes in your contacts run:
 ./geo_code.py -f contacts.db -k <YOUR-KEY> -m update_geo
 where <YOUR-KEY> will look something like (this key is NOT valid!):
   AAAAAAAAABBBBBBBBBBBBCCCCCCC-EEEEE-11111RRRRRRR
 To generate a maemo mapper compatible poi.db file run:
 //geo_code.py -f contacts.db -m dump_sql | sqlite3 poi.db
"""

# python imports
import logging
import optparse
import re
import sys
import time
import urllib
import urllib2
import wconfig
import wpersistance

# available: xml, kml, csv, or json
FORMAT = "xml"
URL = "http://maps.google.com/maps/geo?q=%s&output=%s&key=%s"
LABEL = ""
ACCURACY_THRESHOLD = 8

# ======================================================================
USAGE = 'usage: geo_code [options]*'

OP = optparse.OptionParser(usage=USAGE, conflict_handler='resolve')
OP.add_option('-h', '--help', action='help',
              help='print this help text and exit')
OP.add_option('-k', '--key', dest='key', metavar='KEY',
              help='Google geocode key')
OP.add_option('-m', '--mode', dest='mode', metavar='MODE',
              help='mode: update_geo, dump_sql')
OP.add_option('-f', '--input', dest='input', metavar='WINZIG_DATABASE',
              help='input file name')
OP.add_option('-d', '--debug', action='store_true', dest='debug',
              help='enable debug messages')

CATEGORIES = {
    "club": None,
    "store": None,
    "restaurant": None,
    "private": None,
    "family": None,
    "work": None,
    "business": None,
    "school": None,
    "medical": None,
    "hotel": None,
    "dance": None,
    "other": None,
    }

def GetGeo(address, format, key):
    url = URL % (urllib.quote_plus(address.encode("utf-8")), format, key)
    req = urllib2.Request(url, None)
    try:
      handle = urllib2.urlopen(req)
    except urllib2.HTTPError, err:
        # Errors occur (like 403, 404, etc)
        if err.code < 200:
            raise err
        else:
            header = {}
            if err.hdrs:
                header = err.hdrs
        return "", header, err.code

    header = handle.info()
    data = handle.read()
    handle.close()
    return data, header, 200

PROLOG = """
BEGIN TRANSACTION;
CREATE TABLE poi (poi_id integer PRIMARY KEY, lat real, lon real, label text, desc text, cat_id integer);
CREATE TABLE category (cat_id integer PRIMARY KEY,label text, desc text, enabled integer);
"""
EPILOG = """
COMMIT;
"""


def CleanString(s):
    s = s.replace("'", " ")
    s = s.replace('"', " ")
    s = s.replace('\n', " ")
    return s

def PrintUnicode(s, *args):
    m = s % args
    print m.encode("utf8")
    return

class SqlExport:

    def __init__(self):
        self._count_poi = 0
        self._count_category = 0
        return

    def prolog(self):
        PrintUnicode(PROLOG)
        return

    def epilog(self):
        PrintUnicode(EPILOG)
        return

    def emit_category(self, label, description):
        self._count_category += 1
        label = CleanString(label)
        description = CleanString(description)
        PrintUnicode(u"INSERT INTO 'category' VALUES(%d, '%s', '%s', %d);",
                     self._count_category, label, description, 1)

        return self._count_category

    def emit_poi(self, lattitude, longitude, label, description, cat):
        self._count_poi += 1
        label = CleanString(label)
        description = CleanString(description)
        PrintUnicode(u"INSERT INTO 'poi' VALUES(%d, %s, %s, '%s', '%s', %d);",
                     self._count_poi,
                     lattitude,
                     longitude,
                     label,
                     description,
                     cat)
        return self._count_poi


def GetGeoCodeForRecordFields(rec, lst, key):
    #print rec, lst
    address = [rec[x] for x in lst if rec[x]]
    address = ",".join(address)
    address = re.sub("\[[^\[]+\]", "", address)
    if len(address) < 10:
        return None

    name = rec["n1"]

    data, header, code = GetGeo(address, "csv", key)
    if code != 200:
        logging.error("bad request: %s %s", code, name)
        return None
    code, accuracy, lat, long = data.split(",")
    if int(code) == 610:
        logging.error("bad key")
        return 0
    if int(code) != 200:
        logging.error("bad request2: %s %s", code, name)
        return 0
    if int(accuracy) < ACCURACY_THRESHOLD:
        logging.error("bad accuracy: %s %s", accuracy, name)
        return 0
    logging.info("updating %s", name)
    return lat + " " + long

def UpdateGeoCodes(filename, all, key):
    database = wpersistance.Database(filename)
    ids = database.GetAllIds()
    for id in ids:
        rec = database.RecordLookup(id)
        if all or not rec["g1"]:
            #logging.info("updating %s", rec["n1"])

            geo = GetGeoCodeForRecordFields(rec, ["a1", "a2", "a3"], key)
            if geo:
                rec["g1"] = geo
                database.RecordUpdate(id, rec, 0)
        if all or not rec["g2"]:
            geo = GetGeoCodeForRecordFields(rec, ["a4", "a5", "a6"], key)
            if geo:
                rec["g2"] = geo
                database.RecordUpdate(id, rec, 0)
    database.Write()
    return

def GenerateSqlDump(filename):
    database = wpersistance.Database(filename)
    ids = database.GetAllIds()

    outp = SqlExport()
    logging.info("found %d matches", len(ids))
    outp.prolog()
    for label in CATEGORIES:
      n = outp.emit_category(label, label)
      CATEGORIES[label] = n

    for id in ids:
        rec = database.RecordLookup(id)
        name = rec["n1"]
        remark = rec["r1"]
        cat_no = CATEGORIES["other"]
        for cat in rec["c1"].split():
            if cat in CATEGORIES:
                cat_no = CATEGORIES[cat]
                break

        if rec["g1"] and rec["g1"] not in ["none", "None"]:
            lat, long = rec["g1"].split()
            outp.emit_poi(lat, long, name, remark, cat_no)
        if rec["g2"] and rec["g2"] not in ["none", "None"]:
            lat, long = rec["g2"].split()
            outp.emit_poi(lat, long, name, remark, cat_no)
    outp.epilog()
    logging.info("now run sqlite3 poi.db '.read poi.dump'")
    return

def main(argv_dummy):

    opts, args = OP.parse_args()

    if opts.debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    if not opts.mode:
        logging.error("no mode specified")
        OP.print_help()
        return -1

    if not opts.input:
        logging.error("no database specified")
        OP.print_help()
        return -1

    logging.info("winzig db is %s", opts.input)
    logging.info("google key is is %s", opts.key)

    if opts.mode == "update_geo":
        UpdateGeoCodes(opts.input, 0, opts.key)
    elif opts.mode == "dump_sql":
        GenerateSqlDump(opts.input)
    else:
        logging.error("unknown mode")
        OP.print_help()
        return -1

if __name__ == '__main__':
    sys.exit(main(sys.argv))


