#!/usr/bin/env python
# ======================================================================
# Copyright (C) 2001 and onwards Robert Muth <robert at muth dot org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================

"""
simple vcard generator, using the winzig contact database
but only those entries which have the "sync" label set.
"""

# python imports
import logging
import optparse
import sys

# local imports
import wpersistance

# ======================================================================
USAGE = 'usage: vcards.py [options]*'

OP = optparse.OptionParser(usage=USAGE, conflict_handler='resolve')
OP.add_option('-h', '--help', action='help',
              help='print this help text and exit')
OP.add_option('-l', '--label', dest='label', metavar='LABEL',
              help='label', default = 'sync')
OP.add_option('-f', '--input', dest='input', metavar='WINZIG_DATABASE',
              help='input file name')
OP.add_option('-d', '--debug', action='store_true', dest='debug',
              help='enable debug messages')
OP.add_option('-p', '--path', dest='path',
              help='output path', default = 'vcards/%d.vcf')
# ======================================================================

PROLOG="BEGIN:VCARD"
EPILOG="END:VCARD"
VERSION="VERSION:2.1"

TRANSLATE=[("n1", "N", 1),
           ("n1", "FN", 1),
           ("p1", "TEL;HOME", 0),
           ("m1", "TEL;CELL", 0),
           ]

def MakeCard(rec):
    name = rec.get("n1")
    home = rec.get("p1")
    mobile =  rec.get("m1")

    s = []
    s.append(PROLOG)
    s.append(VERSION)

    for a, b, c in TRANSLATE:
        value = rec.get(a)
        if c:
            assert value
        if value:
            s.append(b + ":" + value)
    s.append(EPILOG)
    return "\r\n".join(s)


if __name__ == '__main__':
    opts, args = OP.parse_args()

    if opts.debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    if not opts.input:
        logging.error("no input specified")
        OP.print_help()
        sys.exit(-1)


    logging.info("using db " + opts.input)
    database = wpersistance.Database(opts.input)

    if opts.label:
        logging.info("using label: " + opts.label)
        ids = database.RestrictByLabels([opts.label], 0)
    else:
        ids = database.GetAllIds()

    logging.info("found %d matches", len(ids))
    for id in ids:
        rec = database.RecordLookup(id)
        s = MakeCard(rec)
        logging.debug("processed: \n" + s + "\n")
        fn = opts.path % id
        logging.info("writing [%s] to %s" % (rec["n1"], fn))
        f = open(fn, "wb")
        f.write(s)
        f.close()
