#!/usr/bin/env python
# ======================================================================
# Copyright (C) 2001 and onwards Robert Muth <robert at muth dot org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================

"""
Winzig's Calculator Widget
"""

# python imports
import gobject
import gtk
import math
import re

# local imports
import wutil

# ======================================================================
__all__ = ["CreateWidget"]
# ======================================================================

_RE_OPERATION = re.compile("^[\~\*\+\-\/\%\&\|\@\^]")
_RE_VARIABLE = re.compile("^\$[0-9]+")
_RE_FUNCTION = re.compile("^[a-zA-Z][a-zA-Z0-9]*")
_RE_NUMBER = re.compile("^[xX0-9a-fA-F\.]+")
_RE_BRACKET = re.compile("^[\(\)]")
_RE_WHITESPACE = re.compile("^\s+")

# ======================================================================

tag_value  = "v"
tag_binary_op = "b"
tag_unary_op = "u"
tag_paren_open = "("
tag_paren_close = ")"

class classOperator:

    def __init__(self, op, prec, desc):
        self.operation = op
        self.precedence = prec
        self.description = desc
        return

def operator_id(x):
    return x

def operator_not(x):
    return -1 ^ x

def operator_neg(x):
    return -x

def operator_add(x, y):
    return x + y

def operator_sub(x, y):
    return x - y

def operator_mul(x, y):
    return x * y

def operator_div(x, y):
    return x / y

def operator_mod(x, y):
    return x % y

def operator_pow(x, y):
    return x ** y

# ======================================================================

_BINARY_OPS = {
    "+": classOperator( operator_add,  6, "addition"),
    "-": classOperator( operator_sub,  6, "substraction"),
    "*": classOperator( operator_mul,  5, "multiplication"),
    "/": classOperator( operator_div,  5, "division"),
    "%": classOperator( operator_mod,  5, "modulo"),
    "^": classOperator( operator_pow,  4, "power (left assoc.)"),
    #"|": classOperator( operator.or_,  3, "bitwise or"),
    #"&": classOperator( operator.and_, 2, "bitwise and"),
    #"@": classOperator( operator.xor,  2, "bitwise xor"),
}
# ======================================================================

_UNARY_OPS = {
    "+":    classOperator( operator_id, 0,  "unary plus"),
    "-":    classOperator( operator_neg, 0, "unary minus"),
    "~":    classOperator( operator_not, 0, "bitwise negation"),
    "sin":  classOperator( math.sin, 0,     "sin"),
    "cos":  classOperator( math.cos, 0,     "cos"),
    "tan":  classOperator( math.tan, 0,     "tan"),
    "asin": classOperator( math.asin, 0,    "arcsin"),
    "acos": classOperator( math.acos, 0,    "arccos"),
    "atan": classOperator( math.atan, 0,    "arctan"),
    "exp":  classOperator( math.exp, 0,     "exp (base e)"),
    "log":  classOperator( math.log, 0,     "log (base e)"),
    "log10":  classOperator( math.log10, 0, "log (base 10)"),
    "sqrt": classOperator( math.sqrt, 0,    "sqrt"),
    }

# ======================================================================

def Coerce(value, mode):
    if mode == "float":
        return float(value)
    elif mode == "int32":
        val =  int(value) % (1 << 32)
        if val < 0: val += (1 << 32)
        return val
    elif mode == "long":
        return long(value)
#    elif mode == "COMPLEX":
#        return complex(value)
    else:
        raise SyntaxError("bad mode " + mode )

# ======================================================================
def MakeValue(value, mode):
    return [tag_value, Coerce(value, mode)]

def MakeParenOpen():
    return [tag_paren_open]

def MakeParenClose():
    return [tag_paren_close]

# ======================================================================

def MakeBinaryOp(op):
    return [tag_binary_op, op, _BINARY_OPS[op].precedence]

def IsBinaryOp(op):
    return _BINARY_OPS.has_key(op)


def ApplyBinaryOp(op, val1, val2):
    if not IsBinaryOp(op):
        raise SyntaxError(_("unknown binary op ") + op)

    fun = _BINARY_OPS[op].operation
    try:
        return fun(val1, val2)
    except Exception:
        raise SyntaxError(_("bad binary operator usage for ") + op)

# ======================================================================

def MakeUnaryOp(op):
    return [tag_unary_op, op]

def ApplyUnaryOp(op, val):
    if not _UNARY_OPS.has_key(op):
        raise SyntaxError(_("unknown unary op ") + op)
    try:
        return _UNARY_OPS[op].operation(val)
    except Exception:
        raise SyntaxError(_("bad unary operator usage for ") + op)

# ======================================================================

def GetValue(lst):
    return lst[1]

def GetPriority(lst):
    return lst[2]

def GetOperator(lst):
    return lst[1]

def GetTag(lst):
    return lst[0]

def GetTags(lsts):
    return [GetTag(l) for l in lsts]

# ======================================================================

def Tokenize(expression, mode, value_stack):
    tokens = []

    # we surrond the whole expression by a pair of parenthesis

    tokens.append( MakeParenOpen())
    next_token_could_be_unary = 1

    while expression != "":
        # print "tokenize:  " + expression + "   ", tokens

        match = _RE_OPERATION.search(expression)
        if match != None:
            token = match.group(0)
            if next_token_could_be_unary:
                tokens.append(MakeUnaryOp(token))
            else:
                if not IsBinaryOp(token):
                    raise SyntaxError(_("token is not binary op ") + token)
                tokens.append(MakeBinaryOp(token))
            # print "match: " + match.group(0) + "  end: " + "%d" % match.end()
            expression = expression[match.end():]
            next_token_could_be_unary = 1
            continue

        match = _RE_VARIABLE.search(expression)
        if match != None:
            token = match.group(0)
            var = token[1:]
            index = int(var)
            if index >= len(value_stack):
                raise SyntaxError(_("unknown variable ") + token)
            val = MakeValue(value_stack[index], mode)
            tokens.append( val )
            # print "match: " + match.group(0) + "  end: " + "%d" % match.end()
            expression = expression[match.end():]
            next_token_could_be_unary = 0
            continue

        match = _RE_FUNCTION.search(expression)
        if match != None:
            token = match.group(0)
            if next_token_could_be_unary:
                tokens.append(MakeUnaryOp(token))
            else:
                raise SyntaxError(_("misplaced function ") + token)
            # print "match: " + match.group(0) + "  end: " + "%d" % match.end()
            expression = expression[match.end():]
            next_token_could_be_unary = 1
            continue

        match = _RE_NUMBER.search(expression)
        if match != None:
            token =  match.group(0)
            try:
                val =  MakeValue(int(token, 0), mode)
            except Exception:
                try:
                    val = MakeValue(float(token), mode)
                except Exception:
                    raise SyntaxError(_("malformed number ") + token)

            tokens.append( val )
            # print "match: " + match.group(0) + "  end: " + "%d" % match.end()
            expression = expression[match.end():]
            next_token_could_be_unary = 0
            continue

        match = _RE_BRACKET.search(expression)
        if match != None:
            token = match.group(0)
            # print "match: " + match.group(0) + "  end: " + "%d" % match.end()
            expression = expression[match.end():]
            if token == "(":
                tokens.append( MakeParenOpen())
                next_token_could_be_unary = 1
            else:
                tokens.append(MakeParenClose())
                next_token_could_be_unary = 0
            continue

        match = _RE_WHITESPACE.search(expression)
        if match != None:
            expression = expression[match.end():]
            continue

        raise SyntaxError(_("unknown expression ") + expression)

    tokens.append(MakeParenClose())
    return tokens


# ======================================================================

def TryToReduceStack(stack, mode):

    # we are reducing at the end of the stack

    if GetTags(stack[-3:]) == [tag_paren_open, tag_value, tag_paren_close]:
        #Debug("REDUCE (V)  -> V\n")
        val = stack[-2]
        stack = stack[:-3]
        stack.append(val)
        return (stack, 1)

    if GetTags(stack[-4:]) == [tag_value, tag_binary_op, tag_value, tag_paren_close]:
        #Debug("REDUCE VBV) -> V\'\n")
        top = stack[-1]
        val = ApplyBinaryOp(GetOperator(stack[-3]),
                            GetValue(stack[-4]),
                            GetValue(stack[-2]))
        stack = stack[:-4]
        stack.append(MakeValue(val, mode))
        stack.append(top)
        return (stack, 1)

    if GetTags(stack[-2:]) == [tag_unary_op, tag_value]:
        #Debug("REDUCE UV -> V\'\n")
        val = ApplyUnaryOp(GetOperator(stack[-2]), GetValue(stack[-1]))
        stack = stack[:-2]
        stack.append(MakeValue(val, mode))
        return (stack, 1)

    if GetTags(stack[-4:]) == [tag_value, tag_binary_op, tag_value, tag_binary_op]:
        if GetPriority(stack[-3]) <= GetPriority(stack[-1]):
            #Debug("REDUCE VBVB\' -> V\'B\n")
            top = stack[-1]
            val = ApplyBinaryOp(GetOperator(stack[-3]),
                                GetValue(stack[-4]),
                                GetValue(stack[-2]))
            stack = stack[:-4]
            stack.append(MakeValue(val, mode))
            stack.append(top)
            return (stack, 1)

    return (stack, 0)

# ======================================================================
# ======================================================================

type_int = type(1)
type_float = type(1.0)
type_long = type(1L)
type_complex = type(1.0j)
type_list  = type([])

# ======================================================================
##
##
# ======================================================================

def EvaluateExpression(tokens, mode):

    stack = []

    # we are reducing at the end of the stack
    for t in tokens:
        stack.append(t)
        while len(stack) != 1:
            #Debug("Stack: " + str(stack) + "\n")
            stack, change =  TryToReduceStack(stack, mode)
            if not change: break

    #Debug("Final: " + str(stack) + "\n")

    if len(stack) != 1 or GetTag(stack[-1]) != tag_value:
        raise SyntaxError(_("not a well formed expression"))
    else:
        return GetValue(stack[-1])

# ======================================================================

class classGui:
    def __init__(self):
        self.calc = None
        self.help  = None
        self.model = None
        self.count = 0
        self.value_history = []
        self.history_ = []
        return

    def SetCalcAndHelp(self, calc, hlp):
        self.calc = calc
        self.help = hlp
        return

    def add_to_history(self, expr):
        if expr in self.history_:
            self.history_.remove(expr)
        self.history_.insert(0, expr)
        return

    def history(self):
        return self.history_

    def switch_to_help(self, dummy, dummy2=None):
        self.calc.hide()
        self.help.show()
        return

    def switch_to_calc(self, dummy, dummy2=None):
        self.help.hide()
        self.calc.show()
        return

    def clear_table(self, dummy, dummy2=None):
        self.model.clear()
        self.count = 0
        self.value_history = []
# ======================================================================
##
# ======================================================================
def DumpOperators(ops):
    return "\n".join(
     ["%-5s %d %-20s" % (k, ops[k].precedence, ops[k].description) for k in ops])

HELP = """
<big>Introduction</big>
This calcuclator allows you do enter computations
as an expression (line of text), e.g.:

1 * 3 + 5 * 6 + sin 5

Results of previous computations are saved and
can be used in new computations using a $number notation.
Double clicking on a previous result will insert it
into the current expression.

The formatting of results can be configured by the user
with a printf style notation:
%%s is generic, %%x hexadecimal, etc.

The calculator can operate in a number of different
modes effecting the results of a computation, e.g.
floating point, integer etc.

<big>Binary Operators (with precedence):</big>
<tt>
%s
</tt>

<big>Unary Operators:</big>
<tt>
%s
</tt>

All operations are left associative.
""" % (DumpOperators(_BINARY_OPS), DumpOperators(_UNARY_OPS))


def CreateWidgetHelp(gui):

    vbox = wutil.MyVBox()
    vbox.show()

    hbox = wutil.MyHBox()
    vbox.pack_start(hbox, 0, 0)
    hbox.show()

    button = wutil.MyButton(_(" Done "),
                            _("exit help screen"),
                            gui.switch_to_calc)
    hbox.pack_start(button, 0, 0, 0)
    button.show()

    window = wutil.MyScrolledWindow()
    vbox.pack_start(window)
    window.show()

    text = wutil.MyLabelWithMarkup(HELP)
    text.show()

    window.add_with_viewport(text)

    return vbox

# ======================================================================
##
# ======================================================================

def CreateWidgetCalc(gui):

    vbox = wutil.MyVBox()
    vbox.show()

    ##

    hbox = wutil.MyHBox()
    vbox.pack_start(hbox, 0, 0)
    hbox.show()

    button = wutil.MyButton(_(" Help "),
                            _("Show Help"),
                            gui.switch_to_help)
    hbox.pack_start(button, 0, 0, 0)
    button.show()

    button = wutil.MyButton(_(" Clr Table "),
                            _("Clear table of results"),
                            gui.clear_table)
    hbox.pack_start(button, 0, 0, 0)
    button.show()


    mode = wutil.MyCombo(["float", "int32", "long"], 8, 0,
                         _("select mode of operation (see Help)"))
    hbox.pack_end(mode, 0, 0, 0)
    mode.show()

    label =  wutil.MyLabel(" Mode:")
    hbox.pack_end(label, 0, 0, 0)
    label.show()

    format = wutil.MyCombo(["%s", "%d", "0x%x", "%f", "%e", "%g"], 5, 1,
                           _("select result formating (see Help)"))
    hbox.pack_end(format, 0, 0, 0)
    format.show()

    label =  wutil.MyLabel(" Fmt:")
    hbox.pack_end(label, 0, 0, 0)
    label.show()

    # ======================================================================##
    # FORWARD DECLARATION FOR READABILITY
    # ======================================================================##
    expression = None
    # ======================================================================##

    def ActionBackSpace(dummy_widget):
        entry =  expression.entry
        pos =  entry.get_position()
        entry.delete_text(pos-1, pos)
        return

    def ActionAddToInput(args, s):

        if args:
            if "0" <= s and s <= "9" or s == "." or s == "x":
                pass
            elif "$" == s:
                s = " " + s
            else:
                s = " " + s + " "
        entry =  expression.entry
        entry.set_position( entry.insert_text(s, entry.get_position()))
        return

    # ======================================================================##

    window = wutil.MyScrolledWindow()
    window.set_shadow_type(gtk.SHADOW_ETCHED_IN)
    vbox.pack_start(window)
    window.show()

    gui.vadj = window.get_vadjustment()


    def ActionHistoryClicked(dummy_a, row, dummy_col):
        ActionAddToInput(None, " $%d" % row[0])
        return

    model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)

    gui.model = model

    view = gtk.TreeView(model)

    renderer = gtk.CellRendererText()
    renderer.set_property('xalign', 1.0)
    column = gtk.TreeViewColumn(_("   Value    "), renderer, text=0)
#    column.set_sort_column_id(1)
    column.set_resizable(1)
    view.append_column(column)

    renderer = gtk.CellRendererText()
    renderer.set_property('xalign', 1.0)
    column = gtk.TreeViewColumn(_("   Expression   "), renderer, text=1)
#    column.set_sort_column_id(2)
    column.set_resizable(1)
    view.append_column(column)


    view.connect("row_activated", ActionHistoryClicked)
    view.set_search_column(1)

    view.show()

    window.add(view)


    hbox = wutil.MyHBox()
    vbox.pack_start(hbox, 0, 0)
    hbox.show()

    buttons = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"]
    for b in buttons:
        button = wutil.MyButton(b, "", ActionAddToInput, b)
        hbox.pack_start(button, 1, 1, 0)
        button.show()

    button = wutil.MyButton(_(" BS "),
                            _("delete previous character"),
                            ActionBackSpace)
    hbox.pack_start(button, 1, 1, 0)
    button.show()

    hbox = wutil.MyHBox()
    vbox.pack_start(hbox, 0, 0)
    hbox.show()

    buttons = ["+", "-", "*", "/", "^", "%", "(", "$", ")", "~", ".", "x"]
    for b in buttons:
        button = wutil.MyButton(b, "", ActionAddToInput, b)
        hbox.pack_start(button, 1, 1, 0)
        button.show()

    hbox = wutil.MyHBox()
    vbox.pack_start(hbox, 0, 0)
    hbox.show()

    buttons = ["log", "log10", "exp", "sqrt",
               "sin", "cos", "tan", "asin", "acos", "atan"]
    for b in buttons:
        button = wutil.MyButton(b, "", ActionAddToInput, b)
        hbox.pack_start(button, 1, 1, 0)
        button.show()

    # ======================================================================##


    frame = gtk.Frame(_(" Expression "))
    frame.show()
    vbox.pack_start(frame, 0, 0)

    hbox = wutil.MyHBox()
    hbox.show()
    frame.add(hbox)

    def ActionClearInput(dummy, dummy2=None):
        expression.entry.set_text("")
        return

    button = wutil.MyButton(_(" Clr "),
                            _("clear expression"),
                            ActionClearInput)
    button.show()
    hbox.pack_start(button, 0, 0, 0)


    def ActionEvalInput(dummy_widget, gui):

        e = expression.entry.get_text()
        f = format.entry.get_text()
        m = mode.entry.get_text()

        try:
            tokens =  Tokenize(e, m, gui.value_history)
            result = EvaluateExpression(tokens, m)
        except SyntaxError, ex:
            wutil.gui_error(str(ex) + ":  " + e)
            return

        try:
            value_str =  (f % result)
        except TypeError:
            wutil.gui_error("format and mode setting seem to be incompatible")
            return

        gui.value_history.append(result)
        it = gui.model.append()
        gui.model.set_value(it, 0,  value_str + (" [$%d]" % gui.count) )
        gui.model.set_value(it, 1,  e + " [" + f + "]"  )

        gui.count = gui.count + 1
        gui.vadj.set_value( gui.vadj.upper)

        # add to history

        gui.add_to_history(e)
        expression.liststore.clear()
        for v in gui.history(): expression.liststore.append((v,))

        return

    expression = wutil.MyCombo([], None, 1,
                               _("enter expression"),
                               ActionEvalInput, gui)
    hbox.pack_start(expression)
    expression.show()


    button = wutil.MyButton(_(" Eval "),
                            _("evaluate expression"),
                            ActionEvalInput, gui)
    button.show()
    hbox.pack_start(button, 0, 0, 0)

    return vbox

# ======================================================================
def CreateWidget(dummy_config_id):
    gui = classGui()
    vbox = wutil.MyVBox()
    gui.help = CreateWidgetHelp(gui)
    gui.calc = CreateWidgetCalc(gui)
    vbox.pack_start(gui.help)
    vbox.pack_start(gui.calc)
    gui.help.hide()
    vbox.show()
    return vbox

# ======================================================================
if __name__ == '__main__':
    window = wutil.MyWindow(_('CALCULATOR'))
    window.add(CreateWidget("wcalculator"))
    window.show()
    wutil.CreateAndRunApp(window)

# ======================================================================
## eof
# ======================================================================







