#!/usr/bin/env python
# ======================================================================
# Copyright (C) 2001 and onwards Robert Muth <robert at muth dot org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================

"""
winzig's Calendar
"""

# python imports
import gobject
import gtk

# local imports
import timedate
import wconfig
import wpersistance
import wutil

# ======================================================================

__all__ = ["CreateWidgets"]

# ======================================================================

TYPE_REGULAR = 0
TYPE_WEEK = 1
TYPE_DAY = 2
TYPE_SIXHOURS = 3

class ClassConcreteEvent:
    """
    an abstract event as stored in the database my
    map to several concrete events due to repetition
    """
    def __init__(self, event, start, duration, text, type):
        self._e = event
        self._s = start
        self._d = duration
        self._text = text
        self._type= type
        return

    def get_event(self):
        return self._e

    def get_type(self):
        return self._type

    def get_start(self):
        return self._s

    def contains_date(self,date):
        return timedate.date_range_contains(self._s,self._d,date)

    def view_month(self, week_start_monday):
        if self._type == TYPE_REGULAR:
            return [ timedate.date_to_string_weekday_day(self._s), self._text]
        elif self._type == TYPE_WEEK:
            start = timedate.date_to_string_day_month(self._s)
            end = timedate.date_to_string_day_month(timedate.date_add(self._s,(6,0,0)))
            return [timedate.date_to_string_week(self._s, week_start_monday),
                    start + " - " + end ]
        else:
            assert(None)
            return

    def view_day(self, week_start_monday):
        return [timedate.date_to_string_time(self._s),
                timedate.date_to_string_time(timedate.date_add(self._s, self._d)),
                self._text]

    def view_week(self, week_start_monday):
        if self._type == TYPE_REGULAR:
            return [ timedate.date_to_string_time(self._s), self._text]
        elif self._type == TYPE_DAY:
            return [timedate.date_to_string_weekday(self._s) ,
                    timedate.date_to_string_day_month(self._s)]
        else:
            assert(None)
            return

    def __cmp__(a,b):
        return  timedate.date_compare(a._s,b._s)

# ======================================================================
##
# ======================================================================
def FindFirstDateAfter(start, length, repeat_mode, target):
    if length == timedate.DurationZero:
        length = timedate.DurationInfinite

    end =  timedate.date_add(start, length)
    if end < target: return None


    if repeat_mode == "yearly":
        x = start
        if x < target:             # just a speed optimization
            delta = timedate.date_sub(target, x)
            x = timedate.date_inc_year(x, (delta[0] / 365)-1 )
        while x < end:
             if x >= target: return x
             x = timedate.date_inc_year(x, 1)
        return None

    elif repeat_mode == "weekly":
        x = start
        if x < target:             # just a speed optimization
            delta = timedate.date_sub(target, x)
            x = timedate.date_inc_week(x, (delta[0] / 7)-1 )
        while x < end:
            if x >= target: return x
            x = timedate.date_inc_week(x, 1)
        return None

    elif repeat_mode == "biweekly":
        x = start
        if x < target:             # just a speed optimization
            delta = timedate.date_sub(target, x)
            x = timedate.date_inc_week(x, (delta[0] / 14)-1 )
        while x < end:
            if x >= target: return x
            x = timedate.date_inc_week(x, 1)
        return None

    elif repeat_mode == "daily":
        x = start
        if x < target:           # just a speed optimization
            delta = timedate.date_sub(target, x)
            x = timedate.date_inc_day(x, delta[0]-1 )
        while x < end:
            if x >= target: return x
            x = timedate.date_inc_day(x,1)
        return None

    else:
         wutil.gui_warning("unsupported repeat pattern: " + repeat_mode)
         return None


class ClassAbstractEvent:
    def __init__(self):
        return

    def init_from_db(self,id, record):
        self._id = id
        field = record["s1"]

        self._start = timedate.date_from_string( field )

        field = record["d1"]
        if field != "":
            self._duration = timedate.date_from_string_duration( field )
        else:
            self._duration = timedate.DurationZero

        field = record["l1"]
        if field:
            self._length = timedate.date_from_string_duration(field)
        else:
            self._length = timedate.DurationZero

        self._action = record["a1"]
        self._repeat = record["r1"]
        self._category = record["c1"]
        self._text = record["t1"]
        return

    def init_default(self, start=None):
        self._id = None
        if start:
            self._start = start
        else:
            self._start = timedate.date_now()
        self._duration = timedate.DurationZero
        self._length = timedate.DurationZero

        self._action = ""
        self._repeat = ""
        self._category = ""
        self._text  = ""
        return

    def init_record(self,record):
        return

    def generate_record(self):
        record = Global._database.RecordLookup(-1)
        record["t1"] = self._text
        record["a1"] = self._action
        record["c1"] = self._category
        record["r1"] = self._repeat
        if self._length != timedate.DurationZero:
            record["l1"] = timedate.date_to_string_duration(self._length)
        if self._duration != timedate.DurationZero:
            record["d1"] = timedate.date_to_string_duration(self._duration)
        record["s1"] =  timedate.date_to_string_full( self._start)
        return record



    def translate_to_display_events(self, begin, end, category):
        if category != "all" and category != self._category: return []
        if self._start >= end: return []

        if self._repeat == "":
            if timedate.date_add(self._start,self._duration) < begin: return []
            return [ClassConcreteEvent(self,self._start,self._duration,self._text,TYPE_REGULAR) ]

        l = []
        while 1:
            begin = FindFirstDateAfter(self._start, self._length, self._repeat, begin)
            if begin == None: break
            if begin >= end: break
            l.append(ClassConcreteEvent(self,
                                        begin,
                                        self._duration,
                                        self._text,
                                        TYPE_REGULAR))
            begin = timedate.date_add(begin, timedate.DurationMinute) # guarantee progress

        return l
# ======================================================================
## read database
# ======================================================================

def read_database(database):
      l = []
      ids = database.GetAllIds(None)
      for rid in ids:
           record = database.RecordLookup(rid)
           event = ClassAbstractEvent()
           event.init_from_db(rid, record)
           l.append(event)
      return l

# ======================================================================
##
# ======================================================================

class ClassGlobal:
      def __init__(self):
            self._current_view = None
            return

Global = ClassGlobal()


# ======================================================================
##
# ======================================================================

def SwitchMainWindow(mode, data=None):
      global Global

      header = Global._header
      mview = Global._view_month.get_widget()
      wview = Global._view_week.get_widget()
      dview = Global._view_day.get_widget()
      edit = Global._edit.get_widget()

      #print "switching to ", mode
      header.hide()
      edit.hide()
      wview.hide()
      mview.hide()
      dview.hide()

#      gui.last_mode = gui.view_mode.get_text()
#      if gui.last_mode == mode:
#            return

#      print "switching to ", mode
#      gui.view_mode.set_text(mode)

      def switch_to(x):
          header.show()
          x.get_widget().show()
          Global._current_view = x
          return

      if mode == "Month" :
#            print data, date_to_string_full(data)
            Global._view_month.update_month( data, 0, 0)
            switch_to(Global._view_month)
      elif mode == "Week" :
            Global._view_week.update_week( data, 0)
            switch_to(Global._view_week)
      elif mode == "Day" :
            Global._view_day.update_day( data, 0)
            switch_to(Global._view_day)

      elif mode == "Previous":
            Global._raw_events = read_database( Global._database )
            Global._current_view.update_body()
            switch_to(Global._current_view)


      elif mode == "EditNew":
            event = ClassAbstractEvent()
            event.init_default(data)
            Global._edit.set_record(event,1)
            edit.show()

      elif mode == "EditUpdate":
            Global._edit.set_record(data,0)
            edit.show()

      return

# ======================================================================
##
# ======================================================================

class ClassEditCalendarRecord:
    """
    """

    def ExtractStart(self):
        return timedate.date_from_y_m_d_H_M(self._start_y,
                                            self._start_m,
                                            self._start_d,
                                            self._start_H,
                                            self._start_M)

    def ExtractDuration(self):
        return timedate.date_from_d_H_M(self._duration_d,
                                        self._duration_H,
                                        self._duration_M)

    def ExtractLength(self):
        return timedate.date_from_d_H_M(self._length_d,
                                        0,
                                        0)


    def UpdateStartYear(self, y):
        self._start_y_menu.set_value(int(y))
        self._start_y = int(y)
        self.UpdateStartMonth(self._start_m)
        return


    def UpdateStartMonth(self, m):
        self._start_m_menu.set_value(m)
        self._start_m = int(m)
        y = self._start_y
        self.UpdateStartDay(self._start_d)
        self._calendar.select_month(m, y)
        return

    def UpdateStartDay(self, d):
        y = self._start_y
        m = self._start_m
        days_in_month =  timedate.days_in_month(m, y)
        if d >= days_in_month: d = days_in_month - 1
        self._start_d = d
        self._start_d_menu.set_value(d)
        week_day = timedate.date_to_string_weekday(
              timedate.date_from_y_m_d_H_M(y, m, d, 0, 0))
        self._weekd_day.set_markup(" <b>%s</b> " % week_day)

        cy, cm, cd = self._calendar.get_date()
        if cd != d + 1: self._calendar.select_day(d + 1)
        return

    def UpdateStartHour(self, h):
        self._start_H_menu.set_value(h)
        self._start_H = int(h)
        return

    def UpdateStartMinute(self, m):
        self._start_M_menu.set_value(m)
        self._start_M = int(m)
        return


    def UpdateDurationDay(self, d):
        self._duration_d_menu.set_value(d)
        self._duration_d = int(d)
        return

    def UpdateDurationHour(self, h):
        self._duration_H_menu.set_value(h)
        self._duration_H = int(h)
        return

    def UpdateDurationMinute(self, m):
        self._duration_M_menu.set_value(m)
        self._duration_M = int(m)
        return


    def UpdateLengthDay(self, d):
        self._length_d_menu.set_value(d)
        self._length_d = int(d)
        return

    def __init__(self):

        global Global
        self._new = -1
        self._widget = None
        self._dirty = -1

        self._start_y = 2000
        self._start_m = 1
        self._start_d = 1
        self._start_H = 0
        self._start_M = 0

        self._duration_d =0
        self._duration_H =0
        self._duration_M =0

        self._length_d =0

        self._day_menu_items = []
        self._calendar = None
        vbox = gtk.VBox()


        def ActionChanged(dummy_arg1):
            self.set_dirty(1, self._new)
            return

        # ======================================================================

        hbox = gtk.HBox()
        hbox.show()
        vbox.pack_start(hbox,0,0)

        label = gtk.Label("")
        label.set_markup("<b>Text</b> ")
        hbox.pack_start(label,0)
        label.show()

        entry = gtk.Entry()
        entry.show()
        entry.connect("changed",ActionChanged)
        hbox.pack_start(entry)

        self._text = entry

        # ======================================================================

        hbox = gtk.HBox()
        hbox.show()
        vbox.pack_start(hbox,0,0)

        label = gtk.Label("")
        label.set_markup("<b>Category</b> ")
        hbox.pack_start(label, 0, 0)
        label.show()

        list = Global._database.GetDatabaseAttribute("c1").list()
        combo = wutil.MyCombo(list, 10, 0,
                              _("chose event category"))
        combo.show()
        combo.entry.connect("changed", ActionChanged)

        hbox.pack_start(combo, 0, 0)

        self._category = combo.entry

        # ======================================================================
        hbox = gtk.HBox()
        hbox.show()
        vbox.pack_start(hbox,0,0)
        label = gtk.Label("")
        hbox.pack_start(label,0)
        label.show()
        # ======================================================================

        hbox = gtk.HBox()
        hbox.show()
        vbox.pack_start(hbox,0,0)

        label = gtk.Label("")
        label.set_markup("<b>Start</b> ")
        hbox.pack_start(label,0)
        label.show()

        def ActionToggleCalendar(widget):
            if widget.get_active():
                calendar.show()
            else:
                calendar.hide()
            return

        button = wutil.MyToggleButton(_(" Cal "),
                                      _("show/hide calendar"))
        button.set_active(0)
        hbox.pack_end(button,0,0)
        button.show()
        button.connect("toggled", ActionToggleCalendar)


        def SetStartYear(dummy, y):
            ActionChanged(dummy)
            self.UpdateStartYear(y)
            return

        spin = wutil.MyOptionSelectRange2(2000, -9999, +9999, 1, 15, 1,
                                         _("select year"),
                                         SetStartYear)
        spin.show()
        hbox.pack_start(spin, 0, 0)
        self._start_y_menu = spin

        def SetStartMonth(dummy, m):
            ActionChanged(dummy)
            self.UpdateStartMonth(m)
            return

        pairs = zip(timedate.MonthNames, range(12))
        options = wutil.MyOptionSelect(pairs,
                                     _("select start month"),
                                     SetStartMonth)
        options.show()
        hbox.pack_start(options, 0, 0)
        self._start_m_menu = options

        def SetStartDay(dummy, d):
            ActionChanged(dummy)
            self.UpdateStartDay(d)
            return

        pairs = [(str(x+1), x) for x in range(31)]
        options = wutil.MyOptionSelect(pairs,
                                     _("select start day"),
                                     SetStartDay)
        options.show()
        hbox.pack_start(options, 0, 0)
        self._start_d_menu = options


        label = gtk.Label("")
        label.set_markup(" <b>Day</b>")
        hbox.pack_start(label, 0, 0)
        self._weekd_day = label
        label.show()


        label = gtk.Label("")
        label.set_markup(" <b>at</b> ")
        hbox.pack_start(label,0)
        label.show()

        def SetStartHour(dummy, h):
            ActionChanged(dummy)
            self.UpdateStartHour(h)
            return

        pairs = [(str(x), x) for x in range(24)]
        options = wutil.MyOptionSelect(pairs,
                                     _("select start hour"),
                                     SetStartHour)
        options.show()
        hbox.pack_start(options, 0, 0)
        self._start_H_menu = options


        label = gtk.Label("")
        label.set_markup("<b>:</b>")
        hbox.pack_start(label,0)
        label.show()

        def SetStartMinute(dummy, m):
            ActionChanged(dummy)
            self.UpdateStartMinute(m)
            return

        pairs = [("%02d" % x, x) for x in range(0, 60, 5)]
        options = wutil.MyOptionSelect(pairs,
                                     _("select start minute"),
                                     SetStartMinute)
        options.show()
        hbox.pack_start(options, 0, 0)
        self._start_M_menu = options

        # ======================================================================
        options = 0;
        options = options |  gtk.CALENDAR_SHOW_DAY_NAMES
        options = options |  gtk.CALENDAR_WEEK_START_MONDAY

        calendar = wutil.MyCalendar(options,
                                    _("select start day"))

        def ActionSelectDay(dummy):
            ActionChanged(dummy)
            y, m, d = self._calendar.get_date()
            self.UpdateStartDay(d-1)
            return

#        calendar.show()
        self._calendar = calendar
        vbox.pack_start(calendar,0,0,0)
        calendar.connect("day-selected", ActionSelectDay)

        # ======================================================================
        hbox = gtk.HBox()
        hbox.show()
        vbox.pack_start(hbox,0,0)
        label = gtk.Label("")
        hbox.pack_start(label,0)
        label.show()

        # ======================================================================
        hbox = gtk.HBox()
        hbox.show()
        vbox.pack_start(hbox,0,0)

        label = gtk.Label("")
        label.set_markup("<b>Duration</b> ")
        hbox.pack_start(label,0)
        label.show()

        def SetDurationDay(dummy, d):
            ActionChanged(dummy)
            self.UpdateDurationDay(d)
            return

        spin = wutil.MyOptionSelectRange2(0, 0, 10000, 1, 1, 1,
                                         _("select days"),
                                         SetDurationDay)
        spin.show()
        hbox.pack_start(spin, 0, 0)
        self._duration_d_menu = spin

        label = gtk.Label("")
        label.set_markup("<b>days</b>  ")
        hbox.pack_start(label,0)
        label.show()

        def SetDurationHour(dummy, h):
            ActionChanged(dummy)
            self.UpdateDurationHour(h)
            return

        pairs = [(str(x), x) for x in range(24)]
        options = wutil.MyOptionSelect(pairs,
                                     _("select duration hours"),
                                     SetDurationHour)
        options.show()
        hbox.pack_start(options, 0, 0)
        self._duration_H_menu = options

        label = gtk.Label("")
        label.set_markup("<b>:</b>")
        hbox.pack_start(label, 0, 0)
        label.show()

        def SetDurationMinute(dummy, m):
            ActionChanged(dummy)
            self.UpdateDurationMinute(m)
            return

        pairs = [("%02d" % x, x) for x in range(0, 60, 5)]
        options = wutil.MyOptionSelect(pairs,
                                     _("select duration hours"),
                                     SetDurationMinute)
        options.show()
        hbox.pack_start(options, 0, 0)
        self._duration_M_menu = options

        # ======================================================================
        hbox = gtk.HBox()
        hbox.show()
        vbox.pack_start(hbox,0,0)
        label = gtk.Label("")
        hbox.pack_start(label,0)
        label.show()
        # ======================================================================

        hbox = gtk.HBox()
        vbox.pack_start(hbox,0,0)
        hbox.show()

        label = gtk.Label("")
        label.set_markup("<b>Repeat</b> ")
        hbox.pack_start(label, 0, 0)
        label.show()

        list = [""] + Global._database.GetDatabaseAttribute("r1").list()
        combo = wutil.MyCombo(list, 8, 0,
                              _("chose repeat behavior"))
        combo.show()
        combo.entry.connect("changed",ActionChanged)

        hbox.pack_start(combo, 0, 0)
        self._repeat = combo.entry

        label = gtk.Label("")
        label.set_markup(" <b>for</b> ")
        hbox.pack_start(label, 0 ,0)
        label.show()

        def SetLengthDay(dummy, d):
            ActionChanged(dummy)
            self.UpdateLengthDay(d)
            return

        spin = wutil.MyOptionSelectRange2(0, 0, 10000, 1, 1, 1,
                                         _("select days"),
                                         SetLengthDay)
        spin.show()
        hbox.pack_start(spin, 0, 0)
        self._length_d_menu = spin


        label = gtk.Label("")
        label.set_markup(" <b>days</b> ")
        hbox.pack_start(label, 0, 0)
        label.show()

            # ======================================================================
            ## FIXME: not yet implemented
            # ======================================================================
            # action: what to do when event fires
#            hbox = gtk.HBox()
#            vbox.pack_start(hbox,0,0)
#            hbox.show()
#            label = gtk.Label(" Action " )
#            hbox.pack_start(label)
#            label.show()

        entry = gtk.Entry()
#            hbox.pack_start(entry)
        entry.show()
#            entry.connect("changed",ActionChanged)

        entry.set_width_chars(8)

        self._action = entry
        # ======================================================================
        hbox = gtk.HBox()
        hbox.show()
        vbox.pack_start(hbox,0,0)
        label = gtk.Label("")
        hbox.pack_start(label,0)
        label.show()
        # ======================================================================

        hbox = gtk.HBox()
        vbox.pack_end(hbox,0,0)
        hbox.show()

        def ActionClickedCancel(dummy_widget):
            SwitchMainWindow("Previous")
            return

        button = wutil.MyButton(_(" Cancel "),
                                _("TODO"))
        hbox.pack_start(button,0,0)
        button.show()
        button.connect("clicked", ActionClickedCancel)
        self._cancel_button = button

        def ActionClickedCreate(dummy_widget):
            global Global
            self.get_widgets_contents( self._event)
            record = self._event.generate_record()
            if not Global._database.RecordCreate(record):
                wutil.gui_error("write error in record create")
            else:
                SwitchMainWindow("Previous")
            return

        button = wutil.MyButton(_(" Create "),
                                _("TODO"))
        hbox.pack_start(button,0,0)
        button.show()
        button.connect("clicked", ActionClickedCreate)
        self._create_button = button

        def ActionClickedUpdate(dummy_widget):
            global Global
            self.get_widgets_contents( self._event)
            record = self._event.generate_record()
            try:
                Global._database.RecordUpdate(self._event._id, record)
            except Exception, err:
                wutil.gui_error(str(err))
            else:
                SwitchMainWindow("Previous")
            return

        button = wutil.MyButton(_(" Update "),
                                _("TODO"))
        hbox.pack_start(button,0,0)
        button.show()
        button.connect("clicked", ActionClickedUpdate)
        self._update_button = button

        def ActionClickedDelete(dummy_widget):
              global Global
              Global._database.RecordDelete(self._event._id)
              SwitchMainWindow("Previous")
              return

        button = wutil.MyButton(_(" Delete "),
                            _("TODO"))
        hbox.pack_end(button,0,0)
        button.show()
        button.connect("clicked", ActionClickedDelete)
        self._delete_button = button


        def ActionReset(dummy_widget):
              self.set_widgets_contents(self._event)
              self.set_dirty(0,self._new)
              return

        button = wutil.MyButton(_(" Reset "),
                                _("TODO"))

        hbox.pack_start(button,0,0)
        button.show()
        button.connect("clicked", ActionReset)
        self._reset_button = button

        vbox.show()
        self._widget = vbox
        return

    def get_widget(self):
        return self._widget


    def get_widgets_contents(self,event):
        event._text = self._text.get_text()
        event._repeat = self._repeat.get_text()
        event._Action = self._action.get_text()
        event._category = self._category.get_text()

        event._start = self.ExtractStart()
        event._duration = self.ExtractDuration()
        event._length = self.ExtractLength()
        return

    def set_dirty(self, dirty, new):
        if self._dirty == dirty and self._new == new: return
        self._dirty = dirty
        self._new = new
        if dirty:

            self._reset_button.show()
            self._delete_button.hide()
            self._cancel_button.show()
            if new:
                  self._create_button.show()
                  self._update_button.hide()
            else:
                  self._create_button.hide()
                  self._update_button.show()
        else:
            self._cancel_button.show()
            self._reset_button.hide()
            if new:
                  self._delete_button.hide()
            else:
                  self._delete_button.show()

            self._create_button.hide()
            self._update_button.hide()

            return


    def set_widgets_contents(self, event):
        y,m,d,H,M = timedate.date_to_y_m_d_H_M( event._start )
        self.UpdateStartYear(y)
        self.UpdateStartMonth(m)
        self.UpdateStartDay(d)
        self.UpdateStartHour(H)
        self.UpdateStartMinute(M)

        d,H,M = timedate.date_to_d_H_M(event._duration)
        self.UpdateDurationDay(d)
        self.UpdateDurationHour(H)
        self.UpdateDurationMinute(M)

        d,H,M = timedate.date_to_d_H_M(event._length)
        self.UpdateLengthDay(d)

        self._repeat.set_text(event._repeat)
        self._action.set_text(event._action)
        self._text.set_text(event._text)
        self._category.set_text(event._category)
        return


    def set_record(self, event, new):
        self.set_widgets_contents(event)
        self._event = event
        self._new = new
        self.set_dirty(0, new)
        return

# ======================================================================

class ClassViewerBase:

    def CreateWidgets(self, titles, has_calendar):
        vbox = gtk.VBox()
        self._widget = vbox

        # ======================================================================

        hbox = gtk.HBox()
        vbox.pack_start(hbox,0,0)
        hbox.show()
        self._menu_bar = hbox

        def ActionToggleCalendar(widget):
            if widget.get_active():
                self._calendar.show()
            else:
                self._calendar.hide()
            return

        if has_calendar:
            button = wutil.MyToggleButton(_(" Cal "),
                                          _("show/hide calendar"))
            button.set_active(0)
            hbox.pack_end(button,0,0)
            button.show()
            button.connect("toggled", ActionToggleCalendar)

        # ======================================================================

        if has_calendar:
            options = 0;
            options = options | gtk.CALENDAR_SHOW_DAY_NAMES
            #options = options | gtk.CALENDAR_SHOW_HEADING
            if self._week_start_monday:
                options = options | gtk.CALENDAR_WEEK_START_MONDAY
            options = options | gtk.CALENDAR_SHOW_WEEK_NUMBERS
            options = options | gtk.CALENDAR_WEEK_START_MONDAY

            calendar = wutil.MyCalendar(options,
                                        "")

            calendar.hide()

            def ActionSwitchToDayView(dummy):
               y,m,d = calendar.get_date()
               SwitchMainWindow("Day", timedate.date_from_y_m_d_H_M(y,m,d-1,0,0))
               return

            calendar.connect("day_selected_double_click",ActionSwitchToDayView)


            vbox.pack_start(calendar,0,0,0)
            self._calendar = calendar
        else:
            self._calendar = None

        # ======================================================================


        window = gtk.ScrolledWindow()
        window.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
        window.show()
        vbox.pack_start(window)

        def ActionEditItem(dummy_arg1, row, dummy_col):
            """
            When an item is clicked, we either edit the item or
            in case of a special item change the view
            """

            #print "path: ",row,col
            display_event =  self._events[row[0]]
            if display_event.get_type() == TYPE_REGULAR:
                SwitchMainWindow("EditUpdate", display_event.get_event())
            elif display_event.get_type() == TYPE_SIXHOURS:
                SwitchMainWindow("EditNew", display_event.get_start())
            elif display_event.get_type() == TYPE_DAY:
                SwitchMainWindow("Day", display_event.get_start())
            elif display_event.get_type() == TYPE_WEEK:
                SwitchMainWindow("Week", display_event.get_start())
            return

        # gross hack, could not find a better way to invoke gtk.ListStore
        # first column cotains color

        args = []
        args += [gtk.gdk.Color]
        args += [int]
        args += ([gobject.TYPE_STRING] * len(titles))
        model = apply( gtk.ListStore, args)
        self._model = model

        view = gtk.TreeView(model)

        for index in range(len(titles)):

            renderer = gtk.CellRendererText()
            column = gtk.TreeViewColumn(titles[index], renderer, text=index+2)
            column.set_resizable(1)

            column.add_attribute(renderer,"background-gdk",0)
#            column.add_attribute(renderer,"style",1)
            column.add_attribute(renderer,"weight",1)
            view.append_column(column)


        view.connect("row_activated",ActionEditItem)
        #view.set_search_column(1);
        view.show()

        self._view_widget = view

#        window.add_with_viewport(view)
        window.add(view)

        style = view.get_style()
        self._color_normal = style.white
        self._color_active = style.bg[gtk.STATE_ACTIVE]
        self._color_now = style.bg[gtk.STATE_SELECTED]

        return

    def __init__(self, titles, viewer, has_calendar, week_start_monday):

        self._widget = None
        self._events = []
        self._interval_begin = None
        self._interval_end = None

        self._viewer =  viewer
        self._color_normal = None
        self._color_active = None
        self._color_now = None
        self._week_start_monday = week_start_monday
        self.CreateWidgets(titles, has_calendar)

        return

    def get_widget(self):
        return self._widget

    def get_interval_begin(self):
        return self._interval_begin

    def get_interval_end(self):
        return self._interval_end

    def set_interval(self, start, length):
        self._interval_begin = start
        self._interval_end = timedate.date_add(start,length)
        if self._calendar:
            new = timedate.date_to_y_m_d_H_M(start)
            old = self._calendar.get_date() # uses gtk format (y,m,d+1)

            # without the following test we might get infinite loop
            # triggering "month_changed" events over and over again

            if new[0] != old[0] or new[1] != old[1]:
                  self._calendar.select_month( new[1],new[0])
            self._calendar.select_day( new[2]+1) # use gtk format for day
        return

    def clear_events(self):
        self._events = []
        return

    def sort_events(self):
        self._events.sort()
        return

    def add_events(self,events):
        self._events.extend(events)
        return

    def add_raw_filtered_events(self):
        global Global
        for e in  Global._raw_events:
            self.add_events(e.translate_to_display_events(self._interval_begin,
                                                          self._interval_end,
                                                          Global._category))
        return


    def find_index_of_earliest_event_after(self, date):
        count = 0
        for event in self._events:
            if event.get_start() < date:
                  count += 1
            else:
                  return count
        return -1

    def scroll_to_index(self, dummy_index):
#        print "scroll to ",index
        #self._view_widget.scroll_to_cell((index,), None,1,0,0)
        self._view_widget.scroll_to_point(100,100)
        return

    def redraw_events(self):
        now = timedate.date_now()
        now_active = 0
        self._model.clear()

        for event in self._events:
            if event.get_type() == TYPE_REGULAR:
                if now_active:
                      fields = [self._color_now, 400]
                else:
                      fields = [self._color_normal, 400]
            else:
                  if event.contains_date(now):
                        now_active = 1
                        fields = [self._color_now, 700]
                  else:
                        now_active = 0
                        fields = [self._color_active, 700]
            fields +=  self._viewer(event, self._week_start_monday)
            it = self._model.append()
            for i in range(len(fields)):
                self._model.set_value(it, i, fields[i])
        return


# ======================================================================
##
# ======================================================================


class ClassViewMonth(ClassViewerBase):
    """
    Per Month Viewer
    """

    def __init__(self, week_start_monday):
        ClassViewerBase.__init__(self,
                                 ["  Date  "," Description "],
                                 ClassConcreteEvent.view_month,
                                 1,
                                 week_start_monday)

        # ======================================================================
        menu_bar = self._menu_bar

        def ActionMoveMonth(dummy_widget, offset):
            #print "move", self.get_start_time(), offset
            self.update_month( self.get_interval_begin(), offset, 0)
            return

        def ActionMoveYear(dummy_widget, offset):
            #print "move", self.get_start_time(), offset
            self.update_month( self.get_interval_begin(), 0, offset)
            return


        button = wutil.MyButton(" << ",
                                _("previous year"))

        menu_bar.pack_start(button,0,0)
        button.show()
        button.connect("clicked", ActionMoveYear, -1)

        button = wutil.MyButton(" < ",
                                _("previous month"))

        menu_bar.pack_start(button,0,0)
        button.show()
        button.connect("clicked", ActionMoveMonth, -1)

        label = gtk.Label("")
        menu_bar.pack_start(label, 0, 0)
        label.show()
        self._month_label = label


        button = wutil.MyButton(" > ",
                                _("next month"))
        menu_bar.pack_start(button,0,0)
        button.show()
        button.connect("clicked", ActionMoveMonth, +1)


        button = wutil.MyButton(" >> ",
                                _("next year"))
        menu_bar.pack_start(button,0,0)
        button.show()
        button.connect("clicked", ActionMoveYear, +1)

        # ======================================================================
        calendar = self._calendar

        def ActionScrollToDay(dummy):
            y,m,d = calendar.get_date()
            date = timedate.date_from_y_m_d_H_M(y,m,d-1,0,0)
            index = self.find_index_of_earliest_event_after( date )
            if index >= 0:
                self.scroll_to_index(index)
            return

        calendar.connect("day_selected",ActionScrollToDay)

        def ActionMonthChanged(dummy):
            y,m,d = calendar.get_date()
            self.update_month( timedate.date_from_y_m_d_H_M(y,m,d-1,0,0), 0, 0)
            return

        calendar.connect("month_changed",ActionMonthChanged)


        return


    def update_head(self):
        """
        update the title
        """


        date = self.get_interval_begin()
        self._month_label.set_markup( '<b>' +
                                      timedate.date_to_string_month_year(date)  +
                                      '</b>')
        return


    def add_week_markers(self):
        begin = self.get_interval_begin()
        end  = self.get_interval_end()
        begin = timedate.date_to_beginning_of_week(begin,
                                                      self._week_start_monday)

        l = []
        while begin < end:
            l.append(ClassConcreteEvent(self,
                                        begin,
                                        timedate.DurationWeek,
                                        "",
                                        TYPE_WEEK))
            begin = timedate.date_add(begin, timedate.DurationWeek)
        self.add_events( l )
        return

    def update_body(self):
        self.clear_events()
        self.add_raw_filtered_events()
        self.add_week_markers()
        self.sort_events()
        self.redraw_events()
        return

    def update_month(self, start, moffset, yoffset):
        """
        set a new month to be displayed -- offset allows to easily inc/dec a month
        """
        #assert offset == 0
        y,m,d,H,M = timedate.date_to_y_m_d_H_M(start)
        m +=  moffset
        y +=  yoffset
        while m < 0:
            m += 12
            y -= 1
        while m > 11:
            m -= 12
            y += 1
        start = timedate.date_from_y_m_d_H_M(y, m, 0, 0, 0)
        length = timedate.date_from_d_H_M( timedate.days_in_month(m,y), 0 ,0 )
        self.set_interval(start, length)
        self.update_head()
        self.update_body()
        return

# ======================================================================
##
# ======================================================================

class ClassViewWeek(ClassViewerBase):

    def __init__(self, week_start_monday):
        ClassViewerBase.__init__(self,
                                 [" Time "," Description "],
                                 ClassConcreteEvent.view_week,
                                 1,
                                 week_start_monday)

        # ======================================================================
        menu_bar = self._menu_bar

        def ActionMoveWeek(dummy_widget, offset):
            #print "move", self.get_start_time(), offset
            self.update_week( self.get_interval_begin(), offset)
            return

        button = wutil.MyButton(" << ",
                                _("previous week"))

        menu_bar.pack_start(button, 0, 0)
        button.show()
        button.connect("clicked", ActionMoveWeek,-1)

        label = gtk.Label("Week")
        menu_bar.pack_start(label, 0, 0)
        label.show()
        self._week_label = label

        button = wutil.MyButton(" >> ",
                                _("next week"))
        menu_bar.pack_start(button, 0, 0)
        button.show()
        button.connect("clicked", ActionMoveWeek,+1)


        def ActionGoMonth(dummy_widget):
            SwitchMainWindow("Month", self.get_interval_begin())
            return

        button = wutil.MyButton(_("Month"),
                                _("go to month"))
        menu_bar.pack_end(button,0,0)
        button.show()
        button.connect("clicked", ActionGoMonth)
        self._month_button = button

        # ======================================================================

        calendar = self._calendar

        def ActionDaySelected(dummy):
            y,m,d = calendar.get_date()
            date = timedate.date_from_y_m_d_H_M(y,m,d-1,0,0)
            self.update_week(date, 0)
            return

        calendar.connect("day_selected", ActionDaySelected)

        return

    def update_head(self):
        date = self.get_interval_begin()
        label =  ('<b>' +
                  timedate.date_to_string_week(date, self._week_start_monday) +
                  '</b>' )
        self._week_label.set_markup(label)

        self._month_button.set_label( timedate.date_to_string_month_year(date) )
        return


    def add_day_markers(self):
        begin = self.get_interval_begin()
        end = self.get_interval_end()

        l = []
        while begin < end:
            l.append(ClassConcreteEvent(self,
                                        begin,
                                        timedate.DurationDay,
                                        "",
                                        TYPE_DAY))
            begin = timedate.date_add(begin, timedate.DurationDay)

        self.add_events( l )
        return

    def update_body(self):
        self.clear_events()
        self.add_raw_filtered_events()
        self.add_day_markers()
        self.sort_events()
        self.redraw_events()
        return

    def update_week(self, start, offset):
        start = timedate.date_inc_week(start,  offset )
        start = timedate.date_to_beginning_of_week(start,
                                                   self._week_start_monday)
        if start == self.get_interval_begin(): return

        self.set_interval(start, timedate.DurationWeek)
        self.update_head()
        self.update_body()
        return

# ======================================================================
##
# ======================================================================

class ClassViewDay(ClassViewerBase):

    def __init__(self, week_start_monday):
        ClassViewerBase.__init__(self,
                                 [" Beg "," End "," Description "],
                                 ClassConcreteEvent.view_day,
                                 1,
                                 week_start_monday)

        # ======================================================================
        menu_bar = self._menu_bar

        def ActionMoveDay(dummy_widget, dir):
            self.update_day(self.get_interval_begin(), dir)
            return

        button = wutil.MyButton(" << ",
                                _("prev day"))
        menu_bar.pack_start(button, 0, 0)
        button.show()
        button.connect("clicked", ActionMoveDay, -1)

        label = gtk.Label("Day")
        menu_bar.pack_start(label, 0, 0l)
        label.show()
        self._day_label = label


        button = wutil.MyButton(" >> ",
                                _("next day"))
        menu_bar.pack_start(button, 0, 0)
        button.show()
        button.connect("clicked", ActionMoveDay, +1)



        def ActionGoMonth(dummy_widget):
            SwitchMainWindow( "Month",self.get_interval_begin())
            return

        button = wutil.MyButton(_("Month"),
                                _("go to month"))

        menu_bar.pack_start(button,1,0)
        button.show()
        button.connect("clicked", ActionGoMonth)

        self._month_button = button

        def ActionGoWeek(dummy_widget):
            SwitchMainWindow("Week", self.get_interval_begin())
            return

        button = wutil.MyButton(_("Week"),
                                _("go to week"))
        menu_bar.pack_end(button,0,0)
        button.show()
        button.connect("clicked", ActionGoWeek)
        self._week_button = button

        # ======================================================================

        calendar = self._calendar
        def ActionDaySelected(dummy):
            y,m,d = calendar.get_date()
            date = timedate.date_from_y_m_d_H_M(y,m,d-1,0,0)
            if date == self.get_interval_begin(): return
            self.update_day(date, 0)
            return

        calendar.connect("day_selected", ActionDaySelected)

        return

    def update_head(self):
        date = self.get_interval_begin()
        label =  '<b>' + timedate.date_to_string_weekday_day(date)  + '</b>'
        self._day_label.set_markup(label)
        label = timedate.date_to_string_week(date, self._week_start_monday)
        self._week_button.set_label(label)
        self._month_button.set_label(timedate.date_to_string_month_year(date))
        return

    def add_hour_markers(self):
        begin = self.get_interval_begin()
        end = self.get_interval_end()

        begin = timedate.date_to_beginning_of_day(begin)
        l = []
        while begin < end:
            l.append(ClassConcreteEvent(self,
                                        begin,
                                        timedate.DurationSixHours,
                                        "",
                                        TYPE_SIXHOURS))
            begin = timedate.date_add(begin, timedate.DurationSixHours)

        self.add_events( l )
        return

    def update_body(self):
        self.clear_events()
        self.add_raw_filtered_events()
        self.add_hour_markers()
        self.sort_events()
        self.redraw_events()
        return

    def update_day(self,start, offset):
        delta = (offset,0,0)
        start = timedate.date_add(start, delta)
        start = timedate.date_to_beginning_of_day(start)
        self.set_interval(start,(1,0,0))
        self.update_head()
        self.update_body()
        return

# ======================================================================
##
# ======================================================================

def MyOptionMenu(list,func):
      view = gtk.OptionMenu()
      menu = gtk.Menu()
      for l in list:
            if l:
                  item=gtk.MenuItem(l)
                  item.connect("activate", func, l)
            else:
                  item=gtk.SeparatorMenuItem()
            item.show()
            menu.add(item)

      view.set_menu( menu )
      return view

# ======================================================================
##
# ======================================================================

def CreateHeaderWidget(week_start_monday):
    global Global

    hbox = gtk.HBox()
    hbox.show()
    # ======================================================================

    def ActionSwitchToEditor( dummy):
          SwitchMainWindow("EditNew")
          return

    button = wutil.MyToggleButton(_(" New "),
                                  _("New Event"))

    button.connect('clicked', ActionSwitchToEditor)
    hbox.pack_start(button,0,0)
    button.show()

    # ======================================================================

    # ======================================================================

#    def ActionSwitchCategory( x, y ):
#          global Global
#          Global._category = y
#          SwitchMainWindow("Previous")
#          return

    list = ["all"] + Global._database.GetDatabaseAttribute("c1").list()
    Global._category = list[0]

#    view = MyOptionMenu(list,ActionSwitchCategory)
#    view.show()
#    hbox.pack_start(view,0,0)
#    tool tips.set_tip( view, "Choose Category");

    # ======================================================================

    label = gtk.Label("Time")
    label.show()
    hbox.pack_start(label,1,1)

    def ActionGoTo(dummy_widget, mode):
        SwitchMainWindow(mode, timedate.date_now())
        return

    button = wutil.MyToggleButton(_("Week"),
                                  _("go to week"))
    hbox.pack_end(button,0,0)
    button.show()
    button.connect("clicked", ActionGoTo, "Week")

    week_button = button

    button = wutil.MyToggleButton(_("Month"),
                                  _("go to month"))
    hbox.pack_end(button,0,0)
    button.show()
    button.connect("clicked", ActionGoTo,"Month")

    month_button = button

    button = wutil.MyToggleButton(_("Day"),
                                  _("go to day"))
    hbox.pack_end(button,0,0)
    button.show()
    button.connect("clicked", ActionGoTo,"Day")

    day_button = button

    def TimerInterrupt():
        #print "timer\n"
        now = timedate.date_now()
        label.set_markup( "<b>" + timedate.date_to_string_time(now) + "</b>" )
        day_button.set_label(timedate.date_to_string_weekday_day(now) )
        month_button.set_label(timedate.date_to_string_month_year(now) )
        l = timedate.date_to_string_week(now, week_start_monday)
        week_button.set_label(l)
        try:
              gobject.timeout_add(30000, TimerInterrupt)
        except:
              # deprecated
              gtk.timeout_add(30000, TimerInterrupt)
        return

    TimerInterrupt()

    # the plan is to use the timer only when the widget is mapped
    #def ActionMapping(args,str):
    #    print "mapping",str
    #    return


    #label.connect('map', ActionMapping,"map")
    #label.connect('unmap', ActionMapping,"unmap")
    #label.connect('show', ActionMapping,"show")
    #label.connect('hide', ActionMapping,"hide")

    return hbox

# ======================================================================
##
# ======================================================================

def CreateWidget(config_id, filename=None):
    global Global
    if not filename: filename = wconfig.GetAppConfig(config_id, 'database')
    week_start_monday = wconfig.GetAppConfig(config_id, 'week_start_monday')

    Global._database = wpersistance.Database(filename)

    Global._raw_events = read_database( Global._database )

    vbox = gtk.VBox()

    Global._view_month = ClassViewMonth(week_start_monday)
    vbox.pack_start(Global._view_month.get_widget())

    Global._view_week = ClassViewWeek(week_start_monday)
    vbox.pack_start(Global._view_week.get_widget())

    Global._view_day = ClassViewDay(week_start_monday)
    vbox.pack_start(Global._view_day.get_widget())


    Global._edit = ClassEditCalendarRecord()
    vbox.pack_start( Global._edit.get_widget())

    Global._header = CreateHeaderWidget(week_start_monday)
    vbox.pack_start(Global._header,0,0)

    SwitchMainWindow("Month", timedate.date_now())

    vbox.show()

    return vbox

# ======================================================================
##
# ======================================================================

def Main():
    import sys
    filename = None
    if len(sys.argv) > 1: filename = sys.argv[1]
    window = wutil.MyWindow(_('CALENDAR'))
    window.add(CreateWidget("wcalendar"))
    window.show()
    wutil.CreateAndRunApp(window)
    return

# ======================================================================
if __name__ == '__main__':
      if 1:
           Main()
      else:
          import profile
          import pstats

          profile.run("Main","run.prof")
          p = pstats.Stats("run.prof")
          p.sort_stats('cumulative').print_stats(20)
          p.sort_stats('time').print_stats(20)

# ======================================================================
## eof
# ======================================================================

