#!/usr/bin/env python
# ======================================================================
# Copyright (C) 2001 and onwards Robert Muth <robert at muth dot org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================

"""
Winzig's Contact Manager And General Database Viewer
"""

__all__ = ["CreateWidget"]

# python imports
import gtk
import sys

# local imports
import wpersistance
import databrowser
import wconfig
import wutil
import google_geo_coding

# ======================================================================
# SWITCHER: EDITOR <-> BROWSER
# ======================================================================
class ClassGlobal(object):
    def __init__(self):
        self._editor = None
        self._browser = None
        return

    def SetEditorAndBrowser(self, editor, browser):
        self._editor = editor
        self._browser = browser
        return

    def SwitchToView(self, redraw):
        self._editor.get_widget().hide()
        if redraw:
            self._browser.Redraw()
        self._browser.get_widget().show()
        return

    def SwitchToEditor(self, rid):
        self._editor.set_edit_record(rid)
        self._editor._modified = 0
        self._browser.get_widget().hide()
        self._editor.get_widget().show()
        return

    def GetNextPrevRecord(self, record_id, direction):
        #print "prev next ", record_id, dir
        #print self._list_view._current_ids
        return self._browser.GetNextPrevRecord(record_id, direction)

Global = ClassGlobal()


# ======================================================================
# EDITOR
# ======================================================================
class ContactEditor(databrowser.DataEditorWithForwardBackward):
    def __init__(self, name, db, elecelectric_tags=[], electric_cb=None):
        self._editor_tags = None
        self._entry_list = None
        self._electric_cb = electric_cb
        self._electric_tags = elecelectric_tags

        # this must be last because it sets some of the instance variables
        databrowser.DataEditorWithForwardBackward.__init__(self, name, db)
        return


    def GetAddress(self, tags):
        address = [self._entry_list[tag].get_text() for tag in tags]
        return [a for a in address if a]

    def AddExtraButtons(self, hbox):
        def ActionGeoCode(dummy_widget, dummy_data):
            name = self._entry_list["n1"].get_text()

            address = self.GetAddress(["a1", "a2", "a3"])
            if address:
                #wutil.gui_warning(address)
                geo = google_geo_coding.GetGeoCodeForAddress(name,
                                                             address, None)
                if not geo:
                    wutil.gui_warning("\n".join(["could not geocode"] + address))
                else:
                    self._entry_list["g1"].set_text(geo)
                    self.set_dirty(1)

            address = self.GetAddress(["a4", "a5", "a6"])
            if address:
                #wutil.gui_warning(address)
                geo = google_geo_coding.GetGeoCodeForAddress(name,
                                                             address, None)
                if not geo:
                    wutil.gui_warning("\n".join(["could not geocode"] + address))
                else:
                    self._entry_list["g2"].set_text(geo)
                    self.set_dirty(1)
            return

        button = wutil.MyButton(_("Update Geo"),
                                _("update the geo code fields from address"),
                                ActionGeoCode)
        hbox.pack_start(button, 0, 0)
        button.show()
        return

    def SwitchToBrowser(self, modified):
        Global.SwitchToView(modified)
        return

    def GetNextPrevRecord(self, rid, direction):
        return Global.GetNextPrevRecord(rid, direction)

    def LoadRecordDataIntoWidgets(self, record):
        for tag in self._editor_tags:
            self._entry_list[tag].set_text(record[tag])
        return

    def RetrieveRecordDataFromWidgets(self, record):
        record["d1"] = wutil.get_current_time_string()
        for tag in self._editor_tags:
            record[tag] = self._entry_list[tag].get_text()
        return


    def CreateEditWidgets(self, vbox):
        self._editor_tags = self._db.GetDatabaseAttribute("se").list()

        window = gtk.ScrolledWindow()
        window.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        vbox.pack_start(window)
        window.show()

        def ActionChanged(dummy_widget):
            self.set_dirty(1)
            return

        table = gtk.Table(len(self._editor_tags), 2, 0)
#        table.set_col_spacing(0,0)
#        table.set_col_spacing(1,0)
        table.set_homogeneous(0)
        table.show()
        count = 0

        self._entry_list = {}
        for tag in self._editor_tags:

            attribute = self._db.GetDatabaseAttribute(tag)
            if attribute == None:
                print "missing attribute", tag
                wutil.gui_error(_("missing attribute ") + tag)
                sys.exit(-1)

            def CallBack(dummy_widget, tag):
                self._electric_cb(tag, self._current_record)
                return

            text = attribute.name()

            if attribute.required():
                text += "*"
            if tag in self._electric_tags:
                button = wutil.MyButton(text, None, CallBack, tag)
                button.show()
                label = button.get_children()[0]
                label.set_markup( "<b>" + text + "</b>")
                table.attach(button, 0, 1, count, count+1,
                             gtk.FILL, gtk.SHRINK, 10, 0)
            else:
                label = gtk.Label("")
                label.set_markup( "<b>" + text + "</b>")
                label.show()
                table.attach(label, 0, 1, count, count+1,
                             gtk.SHRINK, gtk.SHRINK, 10, 0)

            lst = attribute.list()
            # we have a "list of choices"
            if lst:
                combo = wutil.MyCombo(lst, None, 0, attribute.name())
                entry = combo.entry
                combo.show()
                table.attach(combo, 1, 2, count, count+1,
                             gtk.EXPAND | gtk.FILL, gtk.SHRINK)
            else:
                entry = wutil.MyEntry(None, 1,
                                      attribute.name(), None)

                entry.show()
                table.attach(entry, 1, 2, count, count+1,
                             gtk.EXPAND | gtk.FILL, gtk.SHRINK)

            self._entry_list[tag] = entry
            entry.connect("changed", ActionChanged)
            count += 1

        window.add_with_viewport(table)
        return


# ======================================================================
# BROWSER
# ======================================================================
class ContactBrowser(databrowser.DataBrowserWithLabels):

    def __init__(self, name, db):
        self._last_start = ""
        databrowser.DataBrowserWithLabels.__init__(self, name, db)
        return

    def SwitchToEditor(self, rid):
        Global.SwitchToEditor(rid)
        return

    def AddExtraRow(self, vbox):
        self._last_start = ""
        hbox = gtk.HBox()
        vbox.pack_start(hbox, 0, 0)
        hbox.show()

        def ActionWidgetBrowserTabClicked(dummy, key):

            if self._last_start == key[0]: self._last_start = key[1]
            elif self._last_start == key[1]: self._last_start = key[2]
            else: self._last_start = key[0]

            sort_column = self._model.get_sort_column_id()[0]
#            print sort_column
#            sort_tag = self._tags[ self._model.get_sort_column_id()[0]]

            it = self._model.get_iter_first()
            while it:
                value = self._model.get_value(it, sort_column).upper()
                if value == "": value = " "
                if value[0] >= self._last_start: break
                it = self._model.iter_next(it)

            if it:
                # TODO() for backward compatibility
                self._treeview.scroll_to_cell(self._model.get_path(it),
                                              None, 1, 0.0, 0.0)
            return

        headings  = ["#ab",
                     "cde",
                     "fgh",
                     "ijk",
                     "lmn",
                     "opq",
                     "rst",
                     "uvw",
                     "xyz"]

        for l in headings:
            button = wutil.MyButton(l,
                                    _("click repeatedly to cycle through first letters"),
                                    ActionWidgetBrowserTabClicked, l.upper())
            hbox.pack_start(button, 1, 1)
            button.show()
        return


# ======================================================================
def CreateWidget(config_id, filename=None):
    if not filename: filename = wconfig.GetAppConfig(config_id, 'database')

    database = wpersistance.Database(filename)


    vbox = gtk.VBox()
    if not database.GetDatabaseAttribute("lt"):
        wutil.gui_warning("your contact db has an obsolete format\n"
                          "you need to edit the file %s, replacing the line:\n"
                          "c1 name(Category) required() list(...)\n"
                          "with the two lines:\n"
                          "lt name(Label Tag) static() value(c1)\n"
                          "c1 name(Category)\n" % filename)




    electric_tags = wconfig.GetAppConfig(config_id, 'electric_tags')
    electric_cb = wconfig.GetAppConfig(config_id, 'electric_cb')

    editor = ContactEditor("contact", database, electric_tags, electric_cb)

    browser = ContactBrowser("contact", database)
    Global.SetEditorAndBrowser(editor, browser)
    vbox.pack_start(browser.get_widget())
    vbox.pack_start(editor.get_widget())

    Global.SwitchToView(1)

    vbox.show()

    return vbox

# ======================================================================
def Main():
    filename = None
    if len(sys.argv) > 1: filename = sys.argv[1]
    window = wutil.MyWindow(_('CONTACTS)'))
    window.add(CreateWidget("wcontacts", filename))
    window.show()

    wutil.CreateAndRunApp(window)
    return

# ======================================================================
if __name__ == '__main__':
    if 1:
        Main()
    else:
        import profile
        import pstats

        profile.run("Main","run.prof")
        p = pstats.Stats("run.prof")
        p.sort_stats('cumulative').print_stats(20)
        p.sort_stats('time').print_stats(20)

# ======================================================================
## eof
# ======================================================================
