#! /usr/bin/env python
# ======================================================================
# Copyright (C) 2001 and onwards Robert Muth <robert at muth dot org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================
"""
Winzig's Database Engine
"""

# python imports
import sys
import os
import getopt
import re


DB_VERSION = "3"
#######################################################################
# Usage
#######################################################################

def Usage():
    print "Usage [test|query|dump] [-d database-file]"

#######################################################################
# Exceptions
#######################################################################

class DatabaseError(Exception):
    """
    """
    def __init__(self, message):
        Exception.__init__(self)
        self._message = message

    def message(self):
        return self._message

    def __str__(self):
        return self._message

#######################################################################
# Record Structures
#######################################################################

def RecordWrite(rec, output, tags):
    for tag in tags:
        value = rec[tag]
        if value:
            for v in value.split("\n"):
                line = tag + " " + v + "\n"
                line = line.encode('utf8')
                output.write(line)

def RecordInitFromRaw(rec, pairs, line, version):
    if version != DB_VERSION:
        for (tag, value) in pairs:
            if rec[tag]:
                raise DatabaseError("Duplicate tag: %s %s in line %d" %
                                    (tag, value, line))
            rec[tag] = value.replace(r"\n", "\n")
    else:
        for (tag, value) in pairs:
            if rec[tag]:
                rec[tag] = rec[tag] + "\n" + value
            else:
                rec[tag] = value
    return

#######################################################################
#
#######################################################################

class ClassAttribute:

    def __init__(self, tag, number):
        self._tag     = tag
        self._number = number

        self._name  = ""
        self._required  =  0
        self._static  = 0
        self._unique  = 0
        self._list = None
        self._value = None
        return

    def set_property(self, p, v):
        if p == "name": self._name = v
        elif p == "static": self._static = 1
        elif p == "required": self._required = 1
        elif p == "unique": self._unique = 1
        elif p == "value": self._value = v
        elif p == "list": self._list = v.split(",")
        else:
            raise DatabaseError("unknown attribute property: " + p)
        return


    def tag(self):
        return self._tag

    def number(self):
        return self._number

    def static(self):
        return self._static

    def name(self):
        return self._name

    def value(self):
        return self._value

    def set_value(self, v):
        self._value = v
        return

    def unique(self):
        return self._unique

    def list(self):
        l = self._list
        if l: l = l[:]
        return l

    def required(self):
        return self._required

    def write(self, output):
        output.write(self._tag)
        output.write(" name(" + self._name + ")")
        if self._required:
            output.write(" required()")
        if self._static:
            output.write(" static()")
        if self._unique:
            output.write(" unique()")
        if self._value:
            output.write(" value(" + self._value + ")")
        if self._list:
            output.write(" list(" + ",".join(self._list) + ")")
        output.write("\n")
        return

#######################################################################
#
#######################################################################
class ClassSchema:
    """
    The schema is a set of (tag, attribute) pairs.
    """

    def __init__(self):
        self._attributes = {}
        return

    def get_attribute(self, tag):
        return self._attributes.get(tag, None)

    def get_all_attributes_tags(self):
        return self._attributes.keys()

    def get_attribute_tags_original_order(self):
        def mycmp(a1, a2):
            return self._attributes[a1].number() - self._attributes[a2].number()

        tags = self._attributes.keys()
        tags.sort(mycmp)
        return tags

    patProperty = re.compile(r'(\w+)\((.*?)\)')

    def add_attribute(self, tag):
        if tag in self._attributes:
            raise DatabaseError("duplicate attribute: " + tag)
        attribute = ClassAttribute(tag, len(self._attributes))
        self._attributes[tag] = attribute
        return attribute

    def init_from_raw(self, raw_attributes, dummy_line):
        for (tag, value) in raw_attributes:
            attribute = self.add_attribute(tag)

            matches = ClassSchema.patProperty.findall(value)
            assert matches
            for t, v in matches:
                attribute.set_property(t, v)
            assert attribute.name()
        return

    def check_record(self, db, record, rid, line):
        def line_info(line):
            if line:
               return " line %d" % line
            else:
               return ""

        for attribute in self._attributes.values():
            if attribute.static():
                continue
            field = record[attribute.tag()]
            if attribute.unique():
                if db.field_value_exist(attribute.tag(), field, rid):
                    raise DatabaseError("duplicate value\n"
                                        "'%s' exsts for field '%s:%s' %s" %
                                        (field,
                                         attribute.tag(),
                                         attribute.name(),
                                         line_info(line)))


            if attribute.required() and field == "":
                raise DatabaseError("missing required attribute\n %s:%s %s" %
                                    (attribute.tag(),
                                     attribute.name(),
                                     line_info(line)))
            if attribute.list():
                if field != "" and not field in attribute.list():
                    raise DatabaseError("value: '%s' not valid for %s:%s %s" %
                                        (field,
                                         attribute.tag(),
                                         attribute.name(),
                                         line_info(line)))

        for tag in record.keys():
            attribute = self.get_attribute(tag)
            if not attribute:
                raise DatabaseError("Unknown tag: %s %s" % (tag, line_info))
            if attribute.static():
                raise DatabaseError("static tag encountered: %s %s" %
                                    (tag, line_info(line)))
        return

    def write(self, output):
        tags = self.get_attribute_tags_original_order()
        for tag in tags:
            self._attributes[tag].write(output)

    def MakeEmptyRecord(self):
        record = {}
        for attribute in self._attributes.values():
            if attribute.static():
                continue
            record[attribute.tag()] = ""
        return record

#######################################################################
#
#######################################################################
def RenameIfExists(src, dst):
    if os.access(dst, os.F_OK):
        os.remove(dst)
    if os.access(src, os.F_OK):
        os.rename(src, dst)
    return

def SaveBackups(filename, num):
    for i in range(num, 0, -1):
        src = filename + ".%d" % i
        dst = filename + ".%d" % (i+1)
        RenameIfExists(src, dst)
    RenameIfExists(filename, filename + ".1")
    return

#######################################################################
#
#######################################################################

class ClassDatabase:

    def __init__(self):
        self._filename = ""
        self._dirty = 0
        self._schema = None
        self._records = []
        return

    def version(self):
        return self.get_attribute("ve").value()

    def num_records(self):
        return len(self._records)

    def set_dirty(self, state=1):
        self._dirty = state

    def update_record(self, rid, record):
        self._schema.check_record(self, record, rid, None)
        self._records[rid] = record.copy()
        self.set_dirty()
        return

    def add_record(self, record, line=None):
        self._schema.check_record(self, record, None, line)
        self._records.append(record.copy())
        self.set_dirty()
        return len(self._records) - 1

    def del_record(self, rid):
        self._records[rid] = None
        self.set_dirty()
        return

    def get_record(self, rid):
        assert self._records[rid]
        return self._records[rid].copy()

    def field_value_exist(self, field, value, skip):
        for rid in range(len(self._records)):
            if rid == skip: continue
            if self._records[rid] is None: continue
            if self._records[rid][field] == value:
                return 1
        return 0

    def get_record_ids(self, test=None):
        if test is None:
            return [rid for rid in range(len(self._records))
                    if self._records[rid]]
        else:
            return [rid for rid in range(len(self._records))
                    if self._records[rid] and test(self._records[rid])]

    def get_attribute(self, tag):
        return self._schema.get_attribute(tag)


    def read_from_file(self, filename):
        self._filename = filename

        patComment = re.compile(r'^#.*$')
        patEmpty = re.compile(r'^[ \t]*$')
        patTagValue = re.compile(r'^(\w\w)[ ](.*)$')

        try:
            inp = open(filename, "r")
            lines = inp.readlines()
            inp.close()
        except:
            raise DatabaseError("cannot read database " + filename)

        #
        # read set of pairs
        # first set of pairs is scheme
        # remaining set of pairs pairs are records
        #

        line_number = 0
        pairs = []
        for line in lines:

            line_number = line_number + 1
            # utf8
            try:
                line = line.decode('utf8')
            except:
                print "found non utf8 in %s line %d" % (filename, line_number)
                line = line.decode('latin1')
            # skip comments
            match = patComment.search(line)
            if match: continue

            # an empty line represents a record terminator

            match = patEmpty.search(line)
            if match:
                if pairs != []:
                    if self._schema:
                        record = self._schema.MakeEmptyRecord()

                        RecordInitFromRaw(record, pairs, line_number,
                                          self.version())
                        self.add_record(record, line_number)
                    else:
                        self._schema = ClassSchema()
                        self._schema.init_from_raw(pairs, line_number)
                    pairs = []
                continue

            match = patTagValue.search(line)
            if match:
                tag = match.group(1)
                value = match.group(2)
                pairs.append((tag, value))
                continue
            else:
                tag = line.split()[0]
                value = ""
                pairs.append((tag, value))
                continue

            raise DatabaseError("Format error in line: %6d: %s" %
                                (line_number, line[:-1]))

        self.set_dirty(0)
        return

    def write_prolog(self, output):
        output.write("#\n")
        output.write("# Winzig Database\n")
        output.write("# Edit with care!\n")
        output.write("# Use utf-8 encoding!\n")
        output.write("#\n")

    def write_epilog(self, output):
        output.write("#\n")
        output.write("#eof\n")
        output.write("#\n")

    def make_backups(self, n):
        SaveBackups(self._filename, n)
        return

    def file_sort_keys(self):
        sk = self._schema.get_attribute("sk")
        if not sk:
            return sk
        return sk.list()


    def write_to_file(self, sort, filename = None):
        if filename == None:
            filename = self._filename

        output = open(filename, "w")
        self.write_prolog(output)
        self.get_attribute("ve").set_value(DB_VERSION)
        self._schema.write(output)
        output.write("\n")

        tag_order = self._schema.get_attribute_tags_original_order()
        tag_order = [a for a in tag_order
                     if not self.get_attribute(a).static()]

        # this is not a deep copy!
        records = [r for r in self._records if r]
        file_sort_keys = self.file_sort_keys()
        if sort and file_sort_keys:
            def mycmp(r1, r2):
                for tag in file_sort_keys:
                    s1 = r1[tag]
                    s2 = r2[tag]
                    if s1 == s2: continue
                    if s1 < s2: return -1
                    else: return +1
                return 0

            records.sort(mycmp)

        for record in records:
            RecordWrite(record, output, tag_order)
            output.write("\n")

        self.write_epilog(output)
        output.close()
        return
    def MakeEmptyRecord(self):
        return self._schema.MakeEmptyRecord()
#######################################################################
# Main
#######################################################################

def Main(argv):

    # SETTING DEFAULTS

    Config = {}
    Config['database'] = None

    # COMMAND LINE PARSING & ACTION

    assert len(argv) > 1
    mode = argv[1]
    opts, dummy_args = getopt.getopt(argv[2:], "d:")
    for o, a in opts:
        if o == '-d':
            Config['database'] =  a
            continue

    assert Config['database']

    try:
        if mode == 'test':
            db = ClassDatabase()
            db.read_from_file(Config['database'])

        elif mode == 'query':
            db = ClassDatabase()
            db.read_from_file(Config['database'])
            # NYI
            assert 0

        elif mode == 'dump':
            db = ClassDatabase()
            db.read_from_file(Config['database'])
            db.write_to_file(1, "/dev/fd/1")
        else:
            Usage()
            sys.exit(-1)

    except DatabaseError, e:
        print e.message()
        sys.exit(-1)

    sys.exit(0)
#######################################################
#
#######################################################

if __name__ == "__main__": Main(sys.argv)

#######################################################
# eof
#######################################################




