#!/usr/bin/env python
# ======================================================================
# Copyright (C) 2001 and onwards Robert Muth <robert at muth dot org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================

"""
Winzig's Dictionary Widget
"""

# python imports
import re
import gtk

# local imports
import wconfig
import wutil
import urllib
import urllib2

# ======================================================================
kHtmlTags = re.compile("<.*?>")

class DictionaryManager:

    def __init__(self, dictionaries):
        self._dictionaries = dictionaries
        self._hash = {}
        for d in dictionaries:
            self._hash[d[0]] = d
        return

    def CleanupHtml(self, text):
        text = text.replace("&nbsp;", " ")
        text = text.replace("</tr>", "\n")
        text = text.replace("</p>", "\n")
        text = text.replace("</td>", "\t")
        text = text.replace("</th>", "\t")
        return kHtmlTags.sub('', text)

    def names(self):
        return [d[0] for d in self._dictionaries]

    def LookupShell(self, dictionary, words):
        cmd = dictionary[2] % {'words': words}
        (dummy_status, stdout, stderr) = wutil.RunCommand(cmd)
        if stderr:
            return 0, "Error:\n" + stderr
        else:
            #        print len(stdout)
            #        print stderr

            return 1, stdout

    def LookupGet(self, dictionary, words):
        extra = {'User-Agent':
                 'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.7.8) '
                 'Gecko/20050511 Firefox/1.0.4'}

        url = dictionary[2] + urllib.quote_plus(words)
        print url
        req = urllib2.Request(url, None, extra)
        stream = urllib2.urlopen(req)
        page = stream.read()
        try:
            page = page.decode('utf8')
        except Exception:
            page = page.decode('latin1')
        #        print page
        match = re.compile(dictionary[3], re.DOTALL).search(page)
        if not match:
            print url
            print "no match begin ==================================================="
            print page
            print "no match end   ==================================================="
            return 0, "Error:\n" + "regexp for this dictionary seems invalid"
        else:
            #print self.CleanupHtml(match.group(1))
            return 1, self.CleanupHtml(match.group(1))

    def Lookup(self, dictionary, words):
        dictionary = self._hash[dictionary]
        if dictionary[1] == "shell":
            return self.LookupShell(dictionary, words)

        elif dictionary[1] == "get":
            return self.LookupGet(dictionary, words)
        else:
            return 0, "unknown dictionary type: " + dictionary[1]



# ======================================================================
class History:
    def __init__(self, filename=None):
        self._history = []
        if filename:
            for line in file(filename):
                line = line.strip()
                tokens = line.split(",")
                self._history.append(tokens)
            self._file = file(filename, "a")
        else:
            self._file = None
        return

    def Find(self, words, success_only):
        history = []
        for entry in self._history:
            if len(entry) < 3: continue
            if entry[3] != words: continue
            if success_only and int(entry[1]): continue
            history.append(entry)
        return history

    def Append(self, dictionary, status, words):
        date = wutil.get_current_time_string()
        entry = (date, str(status), dictionary, words)
        self._history.append(entry)
        line = "%s,%s,%s,%s\n" % entry
        if self._file:
            self._file.write(line)
            self._file.flush()
        return


template = """Looking up <b>%(words)s</b> in <b>%(dictionary)s</b>:

<b><big>Results:</big></b>
<tt>
%(result)s
</tt>
<b><big>Earlier Lookups:</big></b>
<tt>
%(history)s
</tt>
<span foreground='white' background='white'>
</span>
"""

def CreateWidget(config_id):
    history = History(wconfig.GetAppConfig(config_id, "log"))
    dictionaries = wconfig.GetAppConfig(config_id, "dictionaries")
    maxlinelen = wconfig.GetAppConfig(config_id, "maxlinelen")

    dictionary_manager = DictionaryManager(dictionaries)

    results = None
    query = None
    window = None
    combo = None
    db = None
    clipboard = gtk.Clipboard(selection="PRIMARY")

    word_history = []
    vbox = wutil.MyVBox()
    vbox.show()

    hbox = wutil.MyHBox()
    hbox.show()
    vbox.pack_start(hbox, 0, 0)

    def ActionCopyFromClipboard(dummy, dummy2=None):
        """lookup button was pressed
        """
        text = clipboard.wait_for_text().strip()
        if text:
            combo.entry.set_text(text)
        return

    button = wutil.MyButton(_("Clipboard"),
                            _("Copy current text from clipboard"),
                            ActionCopyFromClipboard)
    hbox.pack_start(button)
    button.show()

    def UpdateDictionary(text):
        """update the big text box with lookup result
        """
        results.set_markup(text)
        return

    def Cleanup(text):
        text = text.replace("<", "&lt;")
        text = text.replace(">", "&gt;")
        return text

    def WrapLongLines(s, maxchars):
        out = []
        for line in s.split("\n"):
            prefix = ""
            while len(line) > maxchars:
                last = line.rfind(" ", maxchars - 10, maxchars)
                if last < 0:
                    last = maxchars-1
                out.append(prefix + line[0:last])
                prefix = " " * 8
                line = line[last:]
            out.append(prefix + line)
        return "\n".join(out)

    def ActionLookup(dummy, dummy2=None):
        """lookup button was pressed
        """
        dictionary = db.entry.get_text()
        words = query.entry.get_text()
        words = words.strip()
        if not words: return

        past_searches = history.Find(words, 1)
        past_searches = ["%s %s\n" % (entry[0], entry[2])
                         for entry in past_searches]

        status, result = dictionary_manager.Lookup(dictionary, words)
        history.Append(dictionary, status, words)
        UpdateDictionary(template %
                         {"words" : words,
                          "dictionary": Cleanup(dictionary),
                          "history": Cleanup("".join(past_searches)),
                          "result": WrapLongLines(result, maxlinelen),
                          })


    #    word_history = word_history[:15]
        word_history.append(words)
        query.liststore.clear()
        for v in word_history:
            query.liststore.append((v,))

        query.entry.set_text("")
        return

    combo = wutil.MyCombo([], None, 1,
                          _("Word(s) to be looked up in a dictionary"))
    hbox.pack_start(combo)
    combo.entry.connect("activate", ActionLookup)
    combo.show()
    query = combo

    button = wutil.MyButton(_("Lookup"), "", ActionLookup)
    hbox.pack_start(button)
    button.show()

    db = wutil.MyCombo(dictionary_manager.names(), None , 0,
                       _("Dictionary to be searched"))
    db.show()
    hbox.pack_end(db, 0, 0)

    window = wutil.MyScrolledWindow()
    vbox.pack_start(window)
    window.show()


    # trickery to keep label in upper left corner
    hbox = wutil.MyHBox()
    hbox.show()
    window.add_with_viewport(hbox)

    # trickery to keep label in upper left corner
    vvbox = wutil.MyVBox()
    vvbox.show()
    hbox.pack_start(vvbox, 0, 0)

    label = wutil.MyLabel("")
    label.show()
    vvbox.pack_start(label, 0, 0)
    results = label



    UpdateDictionary("<big>Welcome to Winzig's Dictionary</big>\n\n\n\n\n\n\n")
    return vbox

# ======================================================================
if __name__ == '__main__':
    window = wutil.MyWindow(_('DICTIONARY'))
    window.add(CreateWidget("wdict"))
    window.show()
    wutil.CreateAndRunApp(window)
# ======================================================================
# eof
# ======================================================================




