#!/usr/bin/env python
# ======================================================================
# Copyright (C) 2001 and onwards Robert Muth <robert at muth dot org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================
"""
Journal: Winzig's journal/diary module
"""

__all__ = ["CreateWidget"]

# python imports
import gtk
import gobject

# local imports
import databrowser
import timedate
import wconfig
import wpersistance
import wutil

# ======================================================================
TAG_DATE = "d1"
TAG_CONTENT = "c1"
TAG_CONTENT2 = "c2"
TAG_MODTIME = "m1"

# ======================================================================
class ClassGlobal(object):
    def __init__(self):
        self._edit = None
        self._list_view = None
        self._interrupt_id = None
        self._interrupt_dialog = self.MakeDialog()
        return

    def EditorIsActive(self):
        return self._edit.get_widget().flags() & gtk.VISIBLE

    def DialogIsActive(self):
        return self._interrupt_dialog.flags() & gtk.VISIBLE

    def AppendTextToTodaysRecord(self, text):
        # print "ADDING: ", text
        # TODO: eliminate the duplication below
        date = timedate.date_now()
        date = (date[0], 0, 0)
        datestr = timedate.date_to_string_full(date)
        def filter_by_date(record):
            return record[TAG_DATE] == datestr

        rids = self._db.GetAllIds(filter_by_date)
        if rids:
            rid = rids[0]
            record = self._db.RecordLookup(rid)
            record[TAG_CONTENT] += text + "\n"
            record[TAG_MODTIME] = wutil.get_current_time_string()
            self._db.RecordUpdate(rid, record)
        else:
            rid = -1
            record = self._db.RecordLookup(rid)
            record[TAG_DATE] = datestr
            record[TAG_CONTENT] += text + "\n"
            record[TAG_MODTIME] = wutil.get_current_time_string()
            self._db.RecordCreate(record)
            # we added a new record, so update browser window
            self._list_view.Redraw()
        return

    def ShowDialog(self):
        self._interrupt_dialog.show_all()

    def HideDialog(self):
        self._interrupt_dialog.hide()

    def MakeDialog(self):
        """This is a hook for external apps add journal entries
        """
        entry = None
        def ActionButtonPressed(dummy):
            self.HideDialog()
            text = entry.get_text()
            if text:
                self.AppendTextToTodaysRecord(entry.get_text())
                entry.set_text("")
            return

        dialog = wutil.MyDialog(_("Journal Entry"))

        label = gtk.Label(_("Current Activity"))
        dialog.vbox.add(label)

        entry = wutil.MyEntry(30, 1, "", None)
        dialog.vbox.add(entry)
        entry.connect("activate", ActionButtonPressed)

        button = gtk.Button(_("OK"))
        dialog.action_area.add(button)
        button.connect("clicked", ActionButtonPressed)
        return dialog


    def SetEditorAndBrowser(self, editor, browser, db):
        self._db = db
        self._edit = editor
        self._list_view = browser
        return

    def SwitchToView(self, redraw):
        self.HideDialog()
        self._edit.get_widget().hide()
        if redraw:
            self._list_view.Redraw()
        self._list_view.get_widget().show()
        return


    def SwitchToEditor(self, rid, record=None):
        self.HideDialog()
        self._list_view.get_widget().hide()
        self._edit.set_edit_record(rid, record)
        self._edit._modified = 0
        self._edit.get_widget().show()
        return

    def GetNextPrevRecord(self, record_id, direction):
        #print "prev next ", record_id, dir
        #print self._list_view._current_ids
        return self._list_view.GetNextPrevRecord(record_id, direction)

    def TimerInterrupt(self):
        if not self.DialogIsActive() and not self.EditorIsActive():
            self.ShowDialog()
        return 1

    def TimerChanged(self, entry):
        tokens = entry.get_text().split()
        sec = 0
        if len(tokens) > 1:
            while tokens:
                num = float(tokens.pop(0))
                unit = tokens.pop(0)
                if unit == "sec":
                    sec += num
                elif unit == "min":
                    sec += num * 60
                elif unit == "h":
                    sec += num * 3600
                elif unit == "d":
                    sec += num * 3600 * 24
                else:
                    print "bad unit", unit
        if self._interrupt_id:
            gobject.source_remove(self._interrupt_id)
            self._interrupt_id = None
        self.HideDialog()
        if sec > 0:
            self._interrupt_id = gobject.timeout_add(int(sec * 1000),
                                                     self.TimerInterrupt)
        return

_GLOBAL = ClassGlobal() # we set the real values later

# ======================================================================
class JournalBrowser(databrowser.DataBrowser):
    """
    """
    def FixupRow(self, row):
        date = timedate.date_from_string(row[1])
        row[1] = row[1].split()[0] + " " + timedate.date_to_string_weekday(date)
        row[2] = row[2]
        return

    def AddExtraButtons(self, hbox):

        def ActionSwitchToDay(dummy_widget, day_offset):
            date = timedate.date_now()
            date = (date[0] + day_offset, 0, 0)
            datestr = timedate.date_to_string_full(date)
            # scan db for this string
            def filter_by_date(record):
                return record[TAG_DATE] == datestr

            rids = self._db.GetAllIds(filter_by_date)
            if rids:
                _GLOBAL.SwitchToEditor(rids[0])
            else:
                rid = -1
                record = self._db.RecordLookup(rid)
                record[TAG_DATE] = datestr
                _GLOBAL.SwitchToEditor(rid, record)
            return

        button = wutil.MyButton(_(" Yesterday "),
                                _("Go to yesterday's entry"),
                                ActionSwitchToDay, -1)
        hbox.pack_end(button, 0, 0)
        button.show()

        button = wutil.MyButton(_(" Today "),
                                _("Go to today's entry"),
                                ActionSwitchToDay, 0)
        hbox.pack_end(button, 0, 0)
        button.show()

        button = wutil.MyButton(_(" Tomorrow "),
                                _("Go to tomorrows's entry"),
                                ActionSwitchToDay, +1)
        hbox.pack_end(button, 0, 0)
        button.show()

        values = [_("never"), "5 sec", "30 min", "1 h" , "2 h"]
        combo = wutil.MyCombo(values, 8, 0, _("set reminder frequency"))
        combo.entry.connect("changed", _GLOBAL.TimerChanged)
        hbox.pack_end(combo, 0, 0)
        combo.show()

        return
        return

    def SwitchToEditor(self, rid):
        _GLOBAL.SwitchToEditor(rid)
        return

# ======================================================================
class JournalEditor(databrowser.DataEditorWithForwardBackward):
    """
    """
    def __init__(self, name, db):
        self._start_y = None  # a widget will
        self._start_m = 1
        self._start_d = 1

        self._start_m_menu = None
        self._start_d_menu = None
        self._calendar = None
        self._weekd_day = None

        self._content = None
        self._content2 = None
        self._date = None
        # this must be last because it sets some of the instance variables
        databrowser.DataEditorWithForwardBackward.__init__(self, name, db)
        return

    def SwitchToBrowser(self, modified):
        _GLOBAL.SwitchToView(modified)
        return

    def GetNextPrevRecord(self, rid, direction):
        return _GLOBAL.GetNextPrevRecord(rid, direction)

    def ExtractStart(self):
        return timedate.date_from_y_m_d_H_M(int(self._start_y.get_value()),
                                            self._start_m,
                                            self._start_d,
                                            0,
                                            0)
    def UpdateStartYear(self, y):
        self._start_y.get_adjustment().set_value(y)
        self.UpdateStartMonth(self._start_m)
        return


    def UpdateStartMonth(self, m):
        self._start_m = m
        self._start_m_menu.set_value(m)
        y = int(self._start_y.get_value())
        self.UpdateStartDay(self._start_d)
        self._calendar.select_month(m, y)
        return

    def UpdateStartDay(self, d):
        y = int(self._start_y.get_value())
        m = self._start_m
        days_in_month =  timedate.days_in_month(m, y)
        if d >= days_in_month: d = days_in_month - 1
        self._start_d = d
        self._start_d_menu.set_value(d)
        week_day = timedate.date_to_string_weekday(
            timedate.date_from_y_m_d_H_M(y, m, d, 0, 0))
        self._weekd_day.set_markup(" <b>%s</b> " % week_day)

        # the calendar might be visible so update it too
        dummy_cy, dummy_cm, cd = self._calendar.get_date()
        if cd != d + 1: self._calendar.select_day(d + 1)
        return

    def LoadRecordDataIntoWidgets(self, record):
        date = record[TAG_DATE]
        if not date: date = wutil.get_current_time_string()

        date = timedate.date_from_string(date)
        y, m, d, dummy_H, dummy_M = timedate.date_to_y_m_d_H_M(date)
        self.UpdateStartYear(y)
        self.UpdateStartMonth(m)
        self.UpdateStartDay(d)

        wutil.FillTaggedBufferFromString(self._content, record[TAG_CONTENT])
        wutil.FillTaggedBufferFromString(self._content2, record[TAG_CONTENT2])
        return

    def RetrieveRecordDataFromWidgets(self, record):
        record[TAG_MODTIME] = wutil.get_current_time_string()
        date = self.ExtractStart()
        record[TAG_DATE] = timedate.date_to_string_full(date)
        record[TAG_CONTENT] = wutil.SaveTaggedBufferToString(self._content)
        record[TAG_CONTENT2] = wutil.SaveTaggedBufferToString(self._content2)
        return

    def CreateEditWidgets(self, vbox):

        hbox = wutil.MyHBox()
        hbox.show()
        vbox.pack_start(hbox, 0, 0)

        def ActionToggleCalendar(toggle):
            if toggle.get_active():
                calendar.show()
            else:
                calendar.hide()
            return

        button = wutil.MyToggleButton(_(" Cal "),
                                      _("show/hide calendar"))
        button.set_active(0)
        hbox.pack_end(button, 0, 0)
        button.show()
        button.connect("toggled", ActionToggleCalendar)


        def StartYearChanged(dummy, val):
            self.set_dirty(1)
            self.UpdateStartYear(int(val))
            return

        spin = wutil.MyOptionSelectRange2(2000, -9999, +9999, 1, 15, 1,
                                          _("select year"),
                                          StartYearChanged)
        spin.show()
        hbox.pack_start(spin, 0, 0)
        self._start_y = spin


        def SetStartMonth(dummy, m):
            self.set_dirty(1)
            self.UpdateStartMonth(m)
            return

        pairs = zip(timedate.MonthNames, range(12))
        options = wutil.MyOptionSelect(pairs,
                                     _("select start month"),
                                     SetStartMonth)
        options.show()
        hbox.pack_start(options, 0, 0)
        self._start_m_menu = options


        def SetStartDay(dummy, day):
            self.set_dirty(1)
            self.UpdateStartDay(day)
            return

        pairs = [(str(x+1), x) for x in range(31)]
        options = wutil.MyOptionSelect(pairs,
                                     _("select start day"),
                                     SetStartDay)
        options.show()
        hbox.pack_start(options, 0, 0)
        self._start_d_menu = options



        label = wutil.MyLabelWithMarkup(_(" <b>Day</b>"))
        hbox.pack_start(label, 0, 0)
        self._weekd_day = label
        label.show()

        # ======================================================================
        options = 0
        options = options |  gtk.CALENDAR_SHOW_DAY_NAMES
        options = options |  gtk.CALENDAR_WEEK_START_MONDAY

        calendar = wutil.MyCalendar(options,
                                    _("select start day"))

        def ActionSelectDay(dummy):
            self.set_dirty(1)
            # we have disabled month/year browsing in  the calendar
            dummy_y, dummy_m, d = self._calendar.get_date()
            self.UpdateStartDay(d-1)
            return

        self._calendar = calendar
        vbox.pack_start(calendar, 0, 0, 0)
        calendar.connect("day-selected", ActionSelectDay)
        # ======================================================================

        paned = gtk.VPaned()
        vbox.pack_start(paned, 1, 1)
        paned.show()
        def ActionSetDirty(dummy_widget):
            self.set_dirty(1)
            return


        # content 1
        window = wutil.MyScrolledWindow()
        window.show()
        paned.add1(window)

        text = wutil.MyTextView(None, ActionSetDirty)
        text.show()
        window.add(text)

        self._content = text

        # content 1
        window = wutil.MyScrolledWindow()
        window.show()
        paned.add2(window)

        text = wutil.MyTextView(None, ActionSetDirty)
        text.show()
        window.add(text)

        self._content2 = text
        return

# ======================================================================
def CreateWidget(config_id, filename = None):
    if not filename: filename = wconfig.GetAppConfig(config_id, 'database')

    database = wpersistance.Database(filename)

    vbox = wutil.MyVBox()

    edit_widget = JournalEditor("journal entry", database)
    vbox.pack_start(edit_widget.get_widget())

    browser_widget = JournalBrowser("journal entry", database)
    vbox.pack_start(browser_widget.get_widget())

    _GLOBAL.SetEditorAndBrowser(edit_widget, browser_widget, database)

    _GLOBAL.SwitchToView(1)

    vbox.show()

    return vbox

# ======================================================================
if __name__ == '__main__':
    import sys
    the_filename = None
    if len(sys.argv) > 1:
        the_filename = sys.argv[1]
    window = wutil.MyWindow(_('JOURNAL'))
    window.add(CreateWidget("wjournal", the_filename))
    window.show()
    wutil.CreateAndRunApp(window)
# ======================================================================
## eof
# ======================================================================
