#!/usr/bin/env python
# ======================================================================
# Copyright (C) 2001 and onwards Robert Muth <robert at muth dot org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================

"""
Notes: Winzig's new notebook
"""

# local imports
import databrowser
import wconfig
import wpersistance
import wutil

# ======================================================================
class ClassGlue:
    def __init__(self):
        self._editor = None
        self._browser = None
        return

    def SetEditorAndBrowser(self, editor, browser):
        self._editor = editor
        self._browser = browser
        return

    def SwitchToView(self, redraw):
        self._editor.get_widget().hide()
        if redraw:
            self._browser.Redraw()
        self._browser.get_widget().show()
        return


    def SwitchToEditor(self, rid):
        self._browser.get_widget().hide()
        self._editor.set_edit_record(rid)
        self._editor._modified = 0
        self._editor.get_widget().show()
        return

    def GetNextPrevRecord(self, record_id, direction):
        #print "prev next ", record_id, dir
        #print self._list_view._current_ids
        return self._browser.GetNextPrevRecord(record_id, direction)

# ======================================================================
class NoteBrowser(databrowser.DataBrowserWithLabels):
    """
    """
    def __init__(self, name, db, glue):
        self._new_label= None
        self._title = None
        self._content = None
        self._glue = glue
        databrowser.DataBrowserWithLabels.__init__(self, name, db)
        return

    def SwitchToEditor(self, rid):
        self._glue.SwitchToEditor(rid)
        return

# ======================================================================
class NoteEditor(databrowser.DataEditorWithForwardBackward):
    """
    """
    def __init__(self, name, db, glue):
        self._new_label= None
        self._title = None
        self._content = None
        self._glue = glue
        databrowser.DataEditorWithForwardBackward.__init__(self, name, db)
        return

    def SwitchToBrowser(self, modified):
        self._glue.SwitchToView(modified)
        return

    def GetNextPrevRecord(self, rid, direction):
        return self._glue.GetNextPrevRecord(rid, direction)


    def RetrieveRecordDataFromWidgets(self, record):
        record["m1"] = wutil.get_current_time_string()
        record["t1"] = self._title.get_text()
        labels = []
        for b in self._buttons:
            if b.get_property("visible"):
                labels.append( b.get_label())
        labels.sort()
        record["l1"] = " ".join(labels)
        record["c1"] = wutil.SaveTaggedBufferToString(self._content)
        return


    def LoadRecordDataIntoWidgets(self, record):
        date = record["d1"]
        if not date: date = wutil.get_current_time_string()
        labels = record["l1"].split()
        for b in self._buttons:
            if labels:
                b.set_label(labels.pop(0))
                b.show()
            else:
                b.hide()
        self._title.set_text(record["t1"])

        wutil.FillTaggedBufferFromString(self._content, record["c1"])
        return


    def CreateEditWidgets(self, vbox):
        def ActionSetDirty(dummy_widget):
            self.set_dirty(1)
            return
        # ======================================================================
        # Labels
        # ======================================================================

        hbox = wutil.MyHBox()
        hbox.show()
        vbox.pack_start(hbox, 0, 0)

        def ActionDeleteLabel(button, dummy=None):
            print button.get_label()
            self.set_dirty(1)
            button.hide()
            return

        for dummy in range(50):
            button = wutil.MyButton("",
                                    _("click to remove label"),
                                    ActionDeleteLabel)
            hbox.pack_start(button, 0, 0)
            self._buttons.append(button)

        combo = wutil.MyCombo([""] + self._db.GetAllLabels(), 10, 1,
                              _("enter or select label to add"))
        entry = combo.entry
        self._new_label = entry
        combo.show()

        def ActionAddLabel(dummy, dummy2=None):
            labels = self._new_label.get_text().split()
            for label in labels:
                if not label: continue
                free_button = None
                for b in self._buttons:
                    if not b.get_property('visible'):
                        free_button = b
                        continue
                    if b.get_label() == label:
                        free_button = None # hack
                        break

                if free_button:
                    self.set_dirty(1)
                    free_button.set_label(label)
                    free_button.show()

            return

        combo.entry.connect("activate", ActionAddLabel)

        button = wutil.MyButton(_("Add"),
                                _("click to add label"),
                                ActionAddLabel)
        hbox.pack_end(button, 0, 0)
        button.show()

        hbox.pack_end(combo, 0, 0)

        # ======================================================================
        hbox = wutil.MyHBox()
        hbox.show()
        vbox.pack_start(hbox, 0, 0)

        label = wutil.MyLabelWithMarkup(_("<b>Title:</b> "))
        label.show()
        hbox.pack_start(label, 0, 0)

        entry = wutil.MyEntry(None, 1, None, ActionSetDirty)
        entry.show()
        hbox.pack_start(entry)
        self._title = entry


        # ======================================================================

        window = wutil.MyScrolledWindow()
        window.show()
        vbox.pack_start(window, 1, 1)

        text = wutil.MyTextView(None, ActionSetDirty)
        text.show()
        window.add(text)
        self._content = text

        # ======================================================================
        return

# ======================================================================
def CreateWidget(config_id, filename = None):
    if not filename: filename = wconfig.GetAppConfig(config_id, 'database')

    database = wpersistance.Database(filename)
    glue = ClassGlue()

    vbox = wutil.MyVBox()

    editor = NoteEditor(config_id, database, glue)
    vbox.pack_start(editor.get_widget())

    browser = NoteBrowser(config_id, database, glue)
    vbox.pack_start(browser.get_widget())

    glue.SetEditorAndBrowser(editor, browser)
    glue.SwitchToView(1)

    vbox.show()

    return vbox

# ======================================================================
if __name__ == '__main__':
    import sys
    the_filename = None
    if len(sys.argv) > 1:
        the_filename = sys.argv[1]
    window = wutil.MyWindow(_('NOTES'))
    window.add(CreateWidget("wnotes"))
    window.show()
    wutil.CreateAndRunApp(window)

# ======================================================================
## eof
# ======================================================================
