#!/usr/bin/env python
# ======================================================================
# Copyright (C) 2001 and onwards Robert Muth <robert at muth dot org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================

"""
Winzig's database abstraction layer
"""

# python imports
import logging
import re

# local imports
import wdatabase
import wutil

# ======================================================================
class Database(object):
    def __init__(self, filename):
        logging.info("opening db [%s]", filename)
        self._filename = filename
        self._database = wdatabase.ClassDatabase()
        try:
            self._database.read_from_file(self._filename)
        except wdatabase.DatabaseError, err:
            wutil.gui_error("error reading db %s\n%s" %
                            (self._filename, str(err)))

        self._matched_ids = None
        self._label_field = self.GetDatabaseAttribute("lt")
        if self._label_field:
            self._label_field =  self._label_field.value()
        self._browser_fields = self.GetDatabaseAttribute("sb").list()
        self._titles = [self.GetDatabaseAttribute(x).name()
                        for x in self._browser_fields]
        logging.info("opening db complete")
        return

    def RecordDelete(self, rid):
        logging.info("delete record %d", rid)
        self._database.del_record(rid)
        self.Write()
        if self._matched_ids and rid in self._matched_ids:
            self._matched_ids.remove(rid)
        return

    def RecordUpdate(self, rid, record, write=1):
        logging.info("update record %d", rid)
        self._database.update_record(rid, record)
        if write:
            self.Write()
        return

    def RecordCreate(self, record):
        logging.info("create record")
        rid = self._database.add_record(record)
        self.Write()
        return rid

    def RecordLookup(self, rid):
        if rid < 0:
            return self._database.MakeEmptyRecord()
        else:
            return self._database.get_record(rid)


    def GetLabelHistogram(self, ids, counts):
        assert self._label_field
        for rid in ids:
            record = self._database.get_record(rid)
            for label in  record[self._label_field].split():
                counts[label] = 1 + counts.get(label, 0)
        return

    def MatchRecords(self, regex):
        if regex == "":
            self._matched_ids =  self.GetAllIds()
            return
        #print  tag, text
        regexes = [re.compile(x, re.I) for x in regex.split()]

        def rec_filter(record):
            values = record.values()
            for r in regexes:
                for v in values:
                    if r.search(v):
                        break
                else:
                    return 0
            return 1

        self._matched_ids = self.GetAllIds(rec_filter)

        return

    def GetLabelsAndCounts(self, ids):
        counts = {}
        self.GetLabelHistogram(ids, counts)
        return counts.items()

    def GetTitles(self):
        # TODO: consider user title from database
        return self._titles

    def GetAllLabels(self):
        labels = [t[0] for t in self.GetLabelsAndCounts(self.GetAllIds())]
        labels.sort()
        return labels

    def LookupBrowserRecord(self, rid):
        record = self._database.get_record(rid)
        return [rid] + [record[field] for field in self._browser_fields]

    def GetBrowserRecords(self, ids):
        records = []
        for rid in ids:
            records.append(self.LookupBrowserRecord(rid))
        return records

    def GetAllIds(self, filter=None):
        return self._database.get_record_ids(filter)

    def RestrictByLabels(self, labels, use_match):
        if use_match:
            ids = self._matched_ids
            assert ids != None
        else:
            ids = self.GetAllIds()

        if not labels:
            return ids

        def keep_record(record):
            record_labels =  record[self._label_field].split()
            if len(record_labels) < len(labels): return 0 # speed opt
            for label in  labels:
                if label not in record_labels: return 0
            return 1

        keep = [rid for rid in ids if keep_record(self._database.get_record(rid))]
        return keep

    def GetDatabaseAttribute(self, attribute):
        return  self._database.get_attribute(attribute)


    def Write(self):
        try:
            self._database.make_backups(8)
            self._database.write_to_file(1)
        except wdatabase.DatabaseError, err:
            wutil.gui_error("error writing db %s\n%s" %
                            (self._filename, str(err)))
        return

