#!/usr/bin/env python
# ======================================================================
# Copyright (C) 2001 and onwards Robert Muth <robert at muth dot org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================

"""
Winzig's Todo List Manager
"""

__all__ = ["CreateWidget"]

# ======================================================================

# local includes
import databrowser
import wconfig
import wpersistance
import wutil

# ======================================================================
class ClassGlobal:
    def __init__(self):
        self._edit_view = None
        self._list_view  = None
        return

    def SetEditorAndBrowser(self, editor, browser):
        self._edit_view = editor
        self._list_view = browser
        return

    def SwitchToView(self, redraw):
        self._edit_view.get_widget().hide()
        if redraw:
            self._list_view.Redraw()
        self._list_view.get_widget().show()
        return

    def SwitchToEditor(self, rid):
        self._edit_view.set_edit_record(rid)
        self._edit_view._modified = 0
        self._list_view.get_widget().hide()
        self._edit_view.get_widget().show()
        return

    def GetNextPrevRecord(self, record_id, direction):
        #print "prev next ", record_id, dir
        #print self._list_view._current_ids
        return self._list_view.GetNextPrevRecord(record_id, direction)

Global = ClassGlobal()

# ======================================================================
class TodoBrowser(databrowser.DataBrowser):
    """
    """
    def __init__(self, name, db):
        self._state_closed_string = None
        self._show_closed = None
        self._viewcontext = None
        databrowser.DataBrowser.__init__(self, name, db)

    def SwitchToEditor(self, rid):
        Global.SwitchToEditor(rid)
        return

    def GetCurrentRecordIds(self):
        def filter_closed(record):
            if self._viewcontext != "all":
                if record["c1"] != self._viewcontext:
                    return 0
            if not self._show_closed:
                if record["s1"] == self._state_closed_string:
                    return 0
            return 1
        return self._db.GetAllIds(filter_closed)

    def AddSearch(self, dummy_hbox):
        # no search in this app
        return

    def AddExtraButtons(self, hbox):
        self._state_closed_string = self._db.GetDatabaseAttribute("s1").list()[-1]
        self._show_closed = 0

        def ActionCloseItem(dummy_widget):
            rid = self.GetSelectedId()
            if rid < 0: return
            record = self._db.RecordLookup(rid)

            record["s1"] = self._state_closed_string
            self._db.RecordUpdate(rid, record)
            self.Redraw()
            return

        button = wutil.MyButton(_(" Close "),
                                _("Close Selected Todo Item"))
        hbox.pack_start(button, 0, 0)
        button.connect('clicked', ActionCloseItem,)
        button.show()


        label = wutil.MyLabel(" Context: ")
        hbox.pack_start(label, 0, 0)
        label.show()

        list_contexts = ["all"]
        list_contexts += self._db.GetDatabaseAttribute("c1").list()

        self._viewcontext = list_contexts[0]

        combo = wutil.MyCombo(list_contexts, 8, 0,
                              _("select context filter"))
        hbox.pack_start(combo, 0, 0)

        def ActionViewCategoryChanged(entry):
            self._viewcontext = entry.get_text()
            #print  gui.viewcategory
            self.Redraw()
            return

        combo.show()
        combo.entry.connect("changed", ActionViewCategoryChanged)

        # ======================================================================
        def ActionShowClosedToggled(button):
            self._show_closed = button.get_active()
            self.Redraw()
            return

        button = wutil.MyCheckButton(_("Show Closed"), self._show_closed, 1,
                                     _("toggle display of closed items"))

        hbox.pack_start(button, 0, 0)
        button.show()
        button.connect("clicked", ActionShowClosedToggled)
        return

# ======================================================================
class TodoEditor(databrowser.DataEditorWithForwardBackward):
    """
    """
    def __init__(self, name, db):
        self._widget_context = None
        self._widget_priority = None
        self._widget_state = None
        self._widget_description = None
        self._widget_project = None
        self._first_state_string = None
        self._first_priority_string = None
        # this must be last because it sets some of the instance variables
        databrowser.DataEditorWithForwardBackward.__init__(self, name, db)
        return

    def SwitchToBrowser(self, modified):
        Global.SwitchToView(modified)
        return

    def GetNextPrevRecord(self, rid, direction):
        return Global.GetNextPrevRecord(rid, direction)


    def RetrieveRecordDataFromWidgets(self, record):
        record["m1"] = wutil.get_current_time_string()
        record["c1"] = self._widget_context.get_text()
        record["p1"] = self._widget_priority.get_text()
        record["s1"] = self._widget_state.get_text()
        record["d1"] = self._widget_description.get_text()
        record["q1"] = self._widget_project.get_text()
        return

    def LoadRecordDataIntoWidgets(self, record):
        if not record["s1"]:
            record["s1"] = self._first_state_string
        if not record["p1"]:
            record["p1"] = self._first_priority_string
        self._widget_context.set_text(record["c1"])
        self._widget_priority.set_text(record["p1"])
        self._widget_project.set_text(record["q1"])
        self._widget_state.set_text(record["s1"])
        self._widget_description.set_text(record["d1"])
        return

    def CreateEditWidgets(self, vbox):
        self._first_state_string = self._db.GetDatabaseAttribute("s1").list()[0]
        self._first_priority_string = self._db.GetDatabaseAttribute("p1").list()[0]

        list_categories = self._db.GetDatabaseAttribute("c1").list()
        list_priorities = self._db.GetDatabaseAttribute("p1").list()
        list_states = self._db.GetDatabaseAttribute("s1").list()

        def ActionSetDirty(dummy_widget):
            self.set_dirty(1)
            return

        hbox = wutil.MyHBox()
        vbox.pack_start(hbox, 0, 0)
        hbox.show()

        label = wutil.MyLabel(_("Context: "))
        label.show()
        hbox.pack_start(label, 0, 0)

        combo = wutil.MyCombo(list_categories, 16, 0, _("select context"))
        combo.show()
        combo.entry.connect("changed", ActionSetDirty)
        hbox.pack_start(combo, 0, 0)
        self._widget_context = combo.entry

        # ======================================================================
        hbox = wutil.MyHBox()
        vbox.pack_start(hbox, 0, 0)
        hbox.show()

        label = wutil.MyLabel(_("Priority: "))
        label.show()
        hbox.pack_start(label, 0, 0)

        combo = wutil.MyCombo(list_priorities, 3, 0, _("select priority"))
        combo.show()
        combo.entry.connect("changed", ActionSetDirty)
        hbox.pack_start(combo, 0, 0)
        self._widget_priority = combo.entry

        label = wutil.MyLabel(_(" State: "))
        label.show()
        hbox.pack_start(label, 0, 0)

        combo = wutil.MyCombo(list_states, 10, 0, _("select state"))
        hbox.pack_start(combo, 0, 0)
        combo.show()
        combo.entry.connect("changed", ActionSetDirty)
        self._widget_state = combo.entry

        # ======================================================================
        hbox = wutil.MyHBox()
        vbox.pack_start(hbox, 0, 0)
        hbox.show()

        label = wutil.MyLabel(_("Project: "))
        label.show()
        hbox.pack_start(label, 0, 0)

        entry = wutil.MyEntry(None, 1, _("project"), ActionSetDirty)
        hbox.pack_start( entry, 0, 0)
        entry.show()
        self._widget_project = entry

        # ======================================================================

        entry = wutil.MyEntry(None, 1, _("short description"), ActionSetDirty)
        vbox.pack_start(entry, 0, 0)
        entry.show()
        self._widget_description = entry
        return

# ======================================================================
def CreateWidget(config_id, filename = None):
    if not filename: filename = wconfig.GetAppConfig(config_id, 'database')

    database = wpersistance.Database(filename)

    if not database.GetDatabaseAttribute("m1"):

        wutil.gui_warning("your todo db has an obsolete format\n"
                          "you need to edit the file %s, replacing the line:\n"
                          "f1 name(Fulltext)\n"
                          "with the ine:\n"
                          "m1 name(Modified)\n"
                          "and\n"
                          "sb name(sort browse) static() list(p1,c1,d1,s1,f1)\n"
                          "with\n"
                          "sb name(sort browser) static() list(p1,c1,q1,d1)\n" % filename)

    vbox = wutil.MyVBox()

    editor = TodoEditor("todo item", database)
    browser = TodoBrowser("todo item", database)

    Global.SetEditorAndBrowser(editor, browser)

    vbox.pack_start(editor.get_widget())
    vbox.pack_start(browser.get_widget())

    Global.SwitchToView(1)

    vbox.show()

    return vbox

# ======================================================================
if __name__ == '__main__':
    import sys
    the_filename = None
    if len(sys.argv) > 1:
        the_filename = sys.argv[1]
    window = wutil.MyWindow(_('TODO'))
    window.add(CreateWidget("wtodo"))
    window.show()
    wutil.CreateAndRunApp(window)

# ======================================================================
## eof
# ======================================================================







