#!/usr/bin/env python
# ======================================================================
# Copyright (C) 2001 and onwards Robert Muth <robert at muth dot org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================

"""
Winzig's Utility Functions
"""

# ======================================================================

__all__ = ["MyCalendar",
           "MyCombo",
           "MySpinButton",
           "MyButton",
           "MyToggleButton",
           "MyCheckButton",
           "Func",
           "MyEntry",
           "MyMenuTitle",
           "RunCommand",
           "MyScrolledWindow",
           "MyLabel",
           "MyHBox",
           "MyVBox",
           "MyLabelWithMarkup",
           "FillTaggedBufferFromString",
           "SaveTaggedBufferToString",
           "SetDirty",
           "CreateAndRunApp",
           ]

# ======================================================================
# python import
import gtk
import pango
import gobject

import gettext
import logging
import os
import select
import signal
import sys
import time
# local imports

# for nokia 770/800
try:
    import hildon
    hildon_support = 1
except ImportError:
    hildon_support = 0
# ======================================================================
# NOTE: we have another player thread running
# we need this to make sure it actually runs
gobject.threads_init()

# ======================================================================


_Tooltips = gtk.Tooltips()
_Tooltips.enable()


_MainWindow = None

_NoteBook = None

# init i18n
gettext.install('winzig', 'locale')
#    l = gettext.translation('winzig', 'locale', languages=['de'])
#    l.install()

# ======================================================================
def CreateAndRunApp(app):
    global _MainWindow

    def ActionStateFullScreen(widget, event):
        if event.new_window_state & gtk.gdk.WINDOW_STATE_FULLSCREEN:
            app.window_in_fullscreen = True
        else:
            app.window_in_fullscreen = False
        return

    def ActionKeyFullScreen(widget, event):
        if not event.keyval == gtk.keysyms.F6:
            return
        if app.window_in_fullscreen:
            app.unfullscreen()
        else:
            app.fullscreen()
        return

    app.window_in_fullscreen = False
    app.connect("key-press-event", ActionKeyFullScreen)
    app.connect("window-state-event", ActionStateFullScreen)

    app.connect("delete_event", ConfirmedQuit, app)
    _MainWindow = app
    logging.info("entering gtk.main() loop")
    try:
        gtk.main()
    except KeyboardInterrupt:
        pass
    return

# ======================================================================
def MyTextView(tt, cb=None):
    tag_table = gtk.TextTagTable()

    tag_bold = gtk.TextTag("bold")
    tag_bold.set_property("weight", pango.WEIGHT_BOLD)

    tag_large = gtk.TextTag("large")
    tag_large.set_property("scale", 1.2)

    #        tag_normal = gtk.TextTag("normal")

    tag_table.add(tag_bold)
    tag_table.add(tag_large)

    buffer =  gtk.TextBuffer(tag_table)
    text = gtk.TextView(buffer)
    if cb:
        buffer.connect("changed", cb)
    return text

def MyCalendar(options, tt):
    calendar = gtk.Calendar()
    calendar.display_options(options)
    _Tooltips.set_tip( calendar, tt)
    return calendar

def MyComboObsolete(values, width, editable, tt):
    combo = gtk.Combo()
    combo.set_popdown_strings(values)
    if width:
        combo.entry.set_width_chars(width)
    combo.set_use_arrows_always(1)
    combo.disable_activate()
    combo.entry.set_editable(editable)
    _Tooltips.set_tip(combo.entry, tt)
    return combo

def MyComboUpdateList(combo, values):
    combo.liststore.clear()
    for v in values:
        combo.liststore.append((v,))
    return

def MyCombo(values, width, editable, tt, cb=None, data=None):
    liststore = gtk.ListStore(str)
    for v in values:
        liststore.append((v,))
    combo = gtk.ComboBoxEntry(liststore, 0)
    combo.liststore = liststore

    if width: combo.child.set_width_chars(width)
    combo.entry = combo.child
    combo.child.set_editable(editable)
    _Tooltips.set_tip(combo.child, tt)
    if values:
        combo.child.set_text(values[0])
    if cb:
        combo.child.connect("activate", cb, data)

    return combo

def MySpinButton(adj, tt, cb=None):
    spin = gtk.SpinButton(adj, 1, 0)
    _Tooltips.set_tip(spin, tt)
    if cb:
        spin.connect("changed", cb)
    if hildon_support:
        spin.set_size_request(60, 50)
    return spin

def MyButton(name="", tt=None, cb=None, cb_data=None):
    button = gtk.Button(name)
    if tt:
        _Tooltips.set_tip(button, tt)
    if cb:
        button.connect("clicked", cb, cb_data)
    return button

def MyToggleButton(name, tt):
    button = gtk.ToggleButton(name)
    _Tooltips.set_tip(button, tt)
    return button

def MyCheckButton(name, active, mode, tt=None, cb=None, cb_data=None):
    button = gtk.CheckButton(name)
    if tt:
        _Tooltips.set_tip(button, tt)
    if cb:
        button.connect("clicked", cb, cb_data)
    button.set_active(active)
    button.set_mode(mode)
    return button


def MyEntry(width, editable, tt, cb):
    entry = gtk.Entry()
    entry.set_editable(editable)
    if width:
        entry.set_width_chars(width)
    if tt:
        _Tooltips.set_tip( entry, tt)
    if cb:
        entry.connect("changed", cb)
    return entry

def MyMenuTitle(name, tt):
    menutitle = gtk.MenuItem(name)
    _Tooltips.set_tip(menutitle, tt)
    return menutitle

def MyLabel(text):
    label = gtk.Label(text)
    return label

def MyLabelWithMarkup(text):
    label = gtk.Label()
    label.set_markup(text)
    return label

def MyScrolledWindow(hpolicy=gtk.POLICY_AUTOMATIC,
                     wpolicy=gtk.POLICY_AUTOMATIC):
    window = gtk.ScrolledWindow()
    window.set_policy(hpolicy, wpolicy)
    return window

def MyWindow(title=None):
    window = gtk.Window()
    if title:
        window.set_title(title)
    return window

def MyVBox():
    return gtk.VBox()

def MyHBox():
    return gtk.HBox()

# ======================================================================
def MyNotebook(pos=gtk.POS_TOP):
    notebook = gtk.Notebook()
    notebook.set_tab_pos(pos)
    return notebook

def SetNotebook(notebook):
    global _NoteBook
    _NoteBook = notebook
    return

def GetNotebook():
    assert _NoteBook
    return _NoteBook


def PopulateNotebookWithWidgets(notebook, apps, reload_pages=0):
    n = -1
    for entry in apps:
        n += 1

        if reload_pages:
            if entry["reload"]:
                notebook.remove_page(n)
            else:
                continue
        logging.info("tab: [%s]", entry["title"])
        logging.info("loading module: [%s]", entry["module"])
        module = __import__(entry["module"])
        logging.info("creating widget")
        widget = module.CreateWidget(entry["config"])
        label = gtk.Label(entry["title"])
        label.set_padding(2, 2)
        notebook.insert_page(widget, label, n)
    return
# ======================================================================
class OptionSelect:

    def __init__(self, pairs, tt, cb):
        self._menubar = gtk.MenuBar()
        self._menubar.show()
        self._pairs = pairs
        self._menutitle = MyMenuTitle("DUMMY", tt)
        self._menutitle.show()
        self._menubar.append(self._menutitle)
        self._menu = gtk.Menu()
        self._menutitle.set_submenu(self._menu)

        for (tag, val) in pairs:
            # the string is important (forces label creation)
            item = gtk.MenuItem(tag)
            item.show()
            if cb:
                item.connect("activate", cb, val)
            self._menu.append(item)
        return

    def set_value(self, value):
        for tag, val in self._pairs:
            if val <= value:
                self._menutitle.get_child().set_text(tag)
        return

def MyOptionSelect(pairs, tt, cb=None):
    options = OptionSelect(pairs, tt, cb)
    menubar = options._menubar
    menubar.set_value  = options.set_value
    return menubar


def MyOptionSelectRange(adj, tt, cb):


    def cb_adaptor(widget):
        cb(widget, adj.get_value())
        return

    spin = MySpinButton(adj, tt, cb_adaptor)

    spin.set_value = adj.set_value
    return spin

def MyOptionSelectRange2(a, b, c, d, e, f, tt, cb):
    if 0 and hildon_support:
        number_editor = hildon.NumberEditor(b, c)
        number_editor.set_value(a)
        def cb_adaptor(widget):
            print "callback"
            cb(widget, number_editor.get_value())
            return

        number_editor.connect("value-changed", cb_adaptor)
        return number_editor

    else:
        adjustment = gtk.Adjustment(a, b, c, d, e, f)

        def cb_adaptor(widget):
            cb(widget, adjustment.get_value())
            return

        spin = MySpinButton(adjustment, tt, cb_adaptor)

        spin.set_value = adjustment.set_value
        return spin
# ======================================================================
def MyDialog(title):
    dialog = gtk.Dialog()
    if _MainWindow:
        dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        dialog.set_transient_for(_MainWindow)
    dialog.set_title(title)
    return dialog

def gui_message(title, message, buttons):
    dialog = MyDialog(title)
    ret = [0]
    label = gtk.Label(message)
    dialog.vbox.add(label)

    def ActionButtonPressed(dummy, a):
        dialog.destroy()
        dialog.response(0)
        ret[0] = a
        if callable(a): a()
        return

    for (b,a) in buttons:
        button = gtk.Button(b)
        dialog.action_area.add(button)
        button.connect("clicked", ActionButtonPressed, a)

    dialog.show_all()
    #dialog.move(500, 500)

    dialog.run()
    return ret[0]

# ======================================================================
#

def gui_error(message):
    gui_message(_("Error"), message, [(_("OK"),None)])
    return
# ======================================================================
#

def gui_warning(message):
    gui_message(_("Warning"), message, [(_("OK"),None)])
    return
# ======================================================================
#

def Error(message):
    print "ERROR: ",message
    sys.exit(-1)

# ======================================================================
_Dirty = {}
# ======================================================================
def SetDirty(app, dirty):
    if dirty:
        _Dirty[app] = 1
    elif app in _Dirty:
        del _Dirty[app]
    return

# ======================================================================
def PopUpDirtyDialog(action="quit"):
    if not _Dirty:
        return 1
    return gui_message("Warning",
                       "Some applications have unsaved data:\n" +
                       "".join(_Dirty.keys()),
                       [(action + " anyway", 1), ("do not " + action, 0)])


def ConfirmedQuit(dummy_widget, dummy_event, window):
    if PopUpDirtyDialog():
        window.destroy()
        gtk.main_quit()
        return 0
    else:
        return 1

# ======================================================================
#

def get_current_time_string():
    return time.strftime("%Y/%m/%d %H:%M", time.localtime(time.time()))

# ======================================================================
# adapted from
# Padraig Brady, 2004/06/24
# http://www.pixelbeat.org/libs/subProcess.py


class SubProcess:
    """Class reresenting a child process. It's like popen2.Popen3
       but there are three main differences.
       1. This makes the new child process group leader (using setpgrp())
          so that all children can be killed.
       2. The output function (read) is optionally non blocking returning in
          specified timeout if nothing is read, or as close to specified
          timeout as possible if data is read.
       3. The output from both stdout & stderr is read (into outdata and
          errdata). Reading from multiple outputs while not deadlocking
          is not trivial and is often done in a non robust manner."""

    def __init__(self, cmd, buffersize):
        """The parameter 'cmd' is the shell command to execute in a
        sub-process. If the 'bufsize' parameter is specified, it
        specifies the size of the I/O buffers from the child process."""
        self._cleaned = 0
        self._buffersize = buffersize
        self._stdout_r, self._stdout_w = os.pipe()
        self._stderr_r, self._stderr_w = os.pipe()
        self._pid = os.fork()
        if self._pid == 0:
            self._child(cmd)
        os.close(self._stdout_w) #parent doesn't write so close
        os.close(self._stderr_w)
        # Note we could use self.stdout=fdopen(self.outr) here
        # to get a higher level file object like popen2.Popen3 uses.
        # This would have the advantages of auto handling the BUFSIZ
        # and closing the files when deleted. However it would mean
        # that it would block waiting for a full BUFSIZ unless we explicitly
        # set the files non blocking, and there would be extra uneeded
        # overhead like EOL conversion. So I think it's handier to use os.read()
        self._stdout_data = ''
        self._stderr_data = ''
        self._stdout_eof = 0
        self._stderr_eof = 0

    def _child(self, cmd):
        # Note sh below doesn't setup a seperate group (job control)
        # for non interactive shells (hmm maybe -m option does?)
        os.setpgrp() #seperate group so we can kill it
        os.dup2(self._stdout_w, 1) #stdout to write side of pipe
        os.dup2(self._stderr_w, 2) #stderr to write side of pipe
        #stdout & stderr connected to pipe, so close all other files
        map(os.close,[self._stdout_r, self._stdout_w, self._stderr_r, self._stderr_w])
        try:
            cmd = ['/bin/sh', '-c', cmd]
            os.execvp(cmd[0], cmd)
        finally: #exit child on error
            os._exit(1)

    def read(self, timeout):
        """
        return 0 when finished
        else return 1 every timeout seconds
        data will be in outdata and errdata
        """

        if timeout != None:
              curent_time = time.time()
              stop_time = curent_time + timeout
        while 1:
            tocheck=[]
            if not self._stdout_eof: tocheck.append(self._stdout_r)
            if not self._stderr_eof: tocheck.append(self._stderr_r)
            if not tocheck: return 0

            ready = select.select(tocheck,[],[],timeout)
            if len(ready[0]) == 0: #no data timeout
                return 1

            if self._stdout_r in ready[0]:
                outchunk = os.read(self._stdout_r,self._buffersize)
                if outchunk == '': self._stdout_eof = 1
                self._stdout_data += outchunk

            if self._stderr_r in ready[0]:
                 errchunk = os.read(self._stderr_r,self._buffersize)
                 if errchunk == '': self._stderr_eof = 1
                 self._stderr_data += errchunk

            if timeout != None:
                current_time =  time.time()
                if current_time > stop_time: return 1 #may be more data but time to go
                timeout = stop_time - current_time

    def status(self): return self._status

    def stderr_data(self): return self._stderr_data

    def stdout_data(self): return self._stdout_data

    def kill(self):
        os.kill(-self._pid, signal.SIGTERM) #kill whole group
        self._status = None
        return

    def cleanup(self):
        """
        Wait for and return the exit status of the child process.
        """
        self._cleaned = 1
        os.close(self._stdout_r)
        os.close(self._stderr_r)
        pid, status = os.waitpid(self._pid, 0)
        if pid == self._pid:
            self._status = status
        return self._status

    def __del__(self):
        if not self._cleaned:
            self.cleanup()
        return

# ======================================================================
def RunCommand(cmd, timeout = None, buffersize = 8192):
    logging.info("running command: [%s]", cmd)
    proc = SubProcess(cmd, buffersize)
    if 0 == proc.read(timeout):
        proc.cleanup()
    else:
        proc.kill()
    return (proc.status(), proc.stdout_data(), proc.stderr_data())

# ======================================================================
def FillTaggedBufferFromString(buffer, fulltext):
    """
    write formatted string to gtk buffer
    also replace newlines
    """
    buffer = buffer.get_buffer()
    it =  buffer.get_start_iter()
    buffer.delete(it,  buffer.get_end_iter() )

    tags = []
    start = 0
    while 1:
        end = fulltext.find(u"@[", start)
        if end < 0: data = fulltext[start:]
        else: data = fulltext[start:end]

        #buffer.insert_with_tags_by_name(iter, data.encode("utf-8"), tags)
        # gross but what do you do if the API is just not right
        apply( buffer.insert_with_tags_by_name, [it, data] + tags)
        if end < 0: break

        start = end + 2
        end = fulltext.find(u"]", start)
        assert( end > 0 )
        for t in fulltext[start:end].split():
            flag = t[0]
            name = t[1:].encode("ascii")
            if flag == "+":
                assert( name not in tags )
                tags.append(name)
            elif flag == '-':
                assert( name in tags )
                tags =  filter( lambda x: x != name, tags)
            else:
                assert 0

        start = end + 1
    return

# ======================================================================
def current_tag_string(it):
    tags_off = [ "-" + x.get_property("name") for x in it.get_toggled_tags(0)]
    tags_on  = [ "+" + x.get_property("name") for x in it.get_toggled_tags(1)]
    tags =  tags_on + tags_off
    if tags: return "@[" + " ".join(tags) + "]"
    else: return ""

# ======================================================================

def SaveTaggedBufferToString(buffer):
    """
    extract formatted string from gtk buffer
    also replace newlines
    """
    buffer = buffer.get_buffer()
    fulltext = ""

    start =  buffer.get_start_iter()
    fulltext += current_tag_string(start)

    while not start.is_end():
        end = start.copy()
        end.forward_to_tag_toggle(None)

        data = buffer.get_text(start, end, 1)
        fulltext += data
        fulltext += current_tag_string(end)
        start = end

    return fulltext


# ======================================================================
## eof
# ======================================================================

